@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2022 releng-tool
 
-__version__ = '0.9.0'
+__version__ = '0.12.0'
 
 # Below should contain a series of helper implementations to assist releng-tool
 # developer's wanting to explicitly import script helpers into scripts not
@@ -22,12 +22,16 @@ __version__ = '0.9.0'
 
 # flake8: noqa
 from os.path import join as releng_join
+from releng_tool.support import releng_include as releng_include
+from releng_tool.support import require_version as releng_require_version
 from releng_tool.util.env import env_value as releng_env
+from releng_tool.util.io import cat as releng_cat
 from releng_tool.util.io import ensure_dir_exists as releng_mkdir
 from releng_tool.util.io import execute as releng_execute
 from releng_tool.util.io import execute_rv as releng_execute_rv
 from releng_tool.util.io import generate_temp_dir as releng_tmpdir
 from releng_tool.util.io import interim_working_dir as releng_wd
+from releng_tool.util.io import ls as releng_ls
 from releng_tool.util.io import path_copy as releng_copy
 from releng_tool.util.io import path_exists as releng_exists
 from releng_tool.util.io import path_move as releng_move
