@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2022 releng-tool
 
 from releng_tool import __version__ as releng_version
 from releng_tool.engine import RelengEngine
@@ -30,11 +30,12 @@ def main():
         parser = argparse.ArgumentParser(
             prog='releng-tool', add_help=False, usage=usage())
 
-        parser.add_argument('action', nargs='?')
+        parser.add_argument('--assets-dir')
         parser.add_argument('--cache-dir')
         parser.add_argument('--debug', action='store_true')
         parser.add_argument('--development', '-D', action='store_true')
         parser.add_argument('--dl-dir')
+        parser.add_argument('--force', '-F', action='store_true')
         parser.add_argument('--help', '-h', action='store_true')
         parser.add_argument('--help-quirks', action='store_true')
         parser.add_argument('--jobs', '-j', default=0, type=type_nonnegativeint)
@@ -76,7 +77,22 @@ def main():
             if sys.platform == 'win32':
                 enable_ansi_win32()
 
-        verbose('releng-tool {}'.format(releng_version))
+        verbose('releng-tool {}', releng_version)
+        debug('({})', __file__)
+
+        # extract additional argument information:
+        #  - pull the action value
+        #  - pull "exec" command (if applicable)
+        #  - key-value entries to be injected into the running
+        #     script/working environment
+        new_args, unknown_args = process_args(unknown_args)
+        args.action = new_args['action']
+        args.action_exec = new_args['exec']
+        args.injected_kv = new_args['entries']
+
+        # register any injected entry into the working environment right away
+        for k, v in args.injected_kv.items():
+            os.environ[k] = v
 
         if unknown_args:
             warn('unknown arguments: {}', ' '.join(unknown_args))
@@ -86,25 +102,14 @@ def main():
 
         # warn if the *nix-based system is running as root; ill-formed projects
         # may attempt to modify the local system's root
-        if sys.platform != 'win32' and os.geteuid() == 0:
-            warn('running as root; this may be unsafe')
+        if sys.platform != 'win32':
+            if os.geteuid() == 0: # pylint: disable=E1101
+                if 'RELENG_IGNORE_RUNNING_AS_ROOT' not in os.environ:
+                    warn('running as root; this may be unsafe')
 
         # prepare engine options
         opts = RelengEngineOptions(args=args, forward_args=forward_args)
 
-        # register the project's root directory as a system path; permits a
-        # project to import locally created modules in their build/etc. scripts
-        sys.path.append(opts.root_dir)
-
-        # register the project's host directory as a system path; lazily permits
-        # loading host tools built by a project over the system
-        host_bin_dir = os.path.join(opts.host_dir + opts.sysroot_prefix, 'bin')
-        sys.path.insert(0, host_bin_dir)
-        os.environ['PATH'] = host_bin_dir + os.pathsep + os.environ['PATH']
-        # support loading applications not following prefix or bin container
-        sys.path.insert(0, opts.host_dir)
-        os.environ['PATH'] = opts.host_dir + os.pathsep + os.environ['PATH']
-
         # create and start the engine
         engine = RelengEngine(opts)
         try:
@@ -114,10 +119,72 @@ def main():
             err(e)
     except KeyboardInterrupt:
         print('')
-        pass
 
     return retval
 
+def process_args(args):
+    """
+    process arguments for an action and key-value entries for environments
+
+    The following will process a remaining argument set for an provided action
+    and entries which could be injected into the script/working environment.
+    The goal is to support Make-styled variable assignment options from a
+    command line, providing users a consistent way to override/set options no
+    matter the platform.
+
+    Args:
+        args: the arguments to check for entries
+
+    Returns:
+        parsed argument options and the remaining/unknown arguments
+    """
+
+    action = None
+    entries = {}
+    exec_ = None
+    needs_exec = False
+    unknown_args = list(args)
+
+    for arg in args:
+        # always ignore option entries
+        if arg.startswith('-'):
+            continue
+
+        if needs_exec:
+            exec_ = arg.strip()
+            unknown_args.remove(arg)
+            debug('detected package-exec call: {}', exec_)
+            needs_exec = False
+
+        is_entry = False
+        if '=' in arg:
+            key, value = arg.split('=', 1)
+            key = key.strip()
+            if key:
+                is_entry = True
+                entries[key] = value.strip()
+                unknown_args.remove(arg)
+                debug('detected entry: {}={}', key, entries[key])
+
+        # if this argument is not an entry and we haven't yet registered an
+        # action yet, consider this argument the action
+        if not action and not is_entry:
+            action = arg
+            unknown_args.remove(arg)
+            debug('detected action: {}', action)
+
+            # if this is assumed to be an `exec`-based package action, consume
+            # the next non-kv entry as the expected command to invoke
+            if action.endswith('-exec'):
+                debug('assuming action is an exec call')
+                needs_exec = True
+
+    return {
+        'action': action,
+        'entries': entries,
+        'exec': exec_,
+    }, unknown_args
+
 def type_nonnegativeint(value):
     """
     argparse type check for a non-negative integer
@@ -164,6 +231,7 @@ def usage():
  <pkg>-clean               clean build directory for package
  <pkg>-configure           perform configure stage for the package
  <pkg>-distclean           pristine clean for package
+ <pkg>-exec <cmd>          invoke a command in the package's directory
  <pkg>-extract             perform extract stage for the package
  <pkg>-fetch               perform fetch stage for the package
  <pkg>-install             perform install stage for the package
@@ -176,6 +244,8 @@ def usage():
  <pkg>-reinstall           force a re-install of a specific package
 
 (common options)
+ --assets-dir <dir>        container directory for download and vcs-cache
+                            directories (e.g. <ASSETS_DIR>/cache)
  --cache-dir <dir>         directory for vcs-cache (default: <ROOT>/cache)
  --dl-dir <dir>            directory for download archives (default: <ROOT>/dl)
  -j, --jobs <jobs>         numbers of jobs to handle (default: 0; automatic)
@@ -190,6 +260,7 @@ def usage():
 
 (other)
  --debug                   show debug-related messages
+ -F, --force               trigger a forced request
  -h, --help                show this help
  --help-quirks             show available quirks
  --nocolorout              explicitly disable colorized output
