@@ -1,22 +1,10 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2022 releng-tool
 
 from collections import OrderedDict
 from datetime import datetime
 from releng_tool import __version__ as releng_version
-from releng_tool.defs import CONF_KEY_CACHE_EXT_TRANSFORM
-from releng_tool.defs import CONF_KEY_DEFINTERN
-from releng_tool.defs import CONF_KEY_EXTENSIONS
-from releng_tool.defs import CONF_KEY_EXTEN_PKGS
-from releng_tool.defs import CONF_KEY_LICENSE_HEADER
-from releng_tool.defs import CONF_KEY_OVERRIDE_REV
-from releng_tool.defs import CONF_KEY_OVERRIDE_SITES
-from releng_tool.defs import CONF_KEY_OVERRIDE_TOOLS
-from releng_tool.defs import CONF_KEY_PKGS
-from releng_tool.defs import CONF_KEY_PREREQUISITES
-from releng_tool.defs import CONF_KEY_QUIRKS
-from releng_tool.defs import CONF_KEY_SYSROOT_PREFIX
-from releng_tool.defs import CONF_KEY_URL_MIRROR
+from releng_tool.defs import ConfKey
 from releng_tool.defs import GlobalAction
 from releng_tool.defs import PkgAction
 from releng_tool.defs import VcsType
@@ -31,17 +19,21 @@ from releng_tool.packages.pipeline import RelengPackagePipeline
 from releng_tool.prerequisites import RelengPrerequisites
 from releng_tool.registry import RelengRegistry
 from releng_tool.stats import RelengStats
+from releng_tool.support import releng_include
+from releng_tool.support import require_version
 from releng_tool.util.env import env_value
 from releng_tool.util.env import extend_script_env
 from releng_tool.util.file_flags import FileFlag
 from releng_tool.util.file_flags import check_file_flag
 from releng_tool.util.file_flags import process_file_flag
 from releng_tool.util.io import FailedToPrepareWorkingDirectoryError
+from releng_tool.util.io import cat
 from releng_tool.util.io import ensure_dir_exists
 from releng_tool.util.io import execute
 from releng_tool.util.io import execute_rv
 from releng_tool.util.io import generate_temp_dir
 from releng_tool.util.io import interim_working_dir
+from releng_tool.util.io import ls
 from releng_tool.util.io import opt_file
 from releng_tool.util.io import path_copy
 from releng_tool.util.io import path_exists
@@ -63,6 +55,8 @@ from releng_tool.util.string import interpret_string
 from releng_tool.util.string import interpret_strings
 from shutil import copyfileobj
 import os
+import ssl
+import sys
 
 class RelengEngine:
     """
@@ -82,8 +76,7 @@ class RelengEngine:
     def __init__(self, opts):
         self.registry = RelengRegistry()
         self.opts = opts
-        self.pkgman = RelengPackageManager(opts, self.registry,
-            dvcs_cache=True)
+        self.pkgman = RelengPackageManager(opts, self.registry, dvcs_cache=True)
         self.stats = RelengStats(opts)
 
     def run(self):
@@ -98,25 +91,59 @@ class RelengEngine:
             issue has occurred when interpreting or running the user's
             configuration/package definitions
         """
+
         opts = self.opts
         gaction = opts.gbl_action
+        license_files = {}
+        pa = opts.pkg_action
+
         if gaction == GlobalAction.INIT:
             return initialize_sample(opts)
 
-        self.start_time = datetime.now().replace(microsecond=0)
-        verbose("loading user's configuration...")
+        start_time = datetime.now().replace(microsecond=0)
         gbls = {
             'releng_args': opts.forward_args,
             'releng_version': releng_version,
         }
 
+        # if any injected key-value entries are provided, attempt to add them
+        # into the global context
+        if opts.injected_kv:
+            debug('injecting key-value arguments into global context')
+            gbls.update(opts.injected_kv)
+
         debug('loading statistics...')
         self.stats.load()
 
+        # inform the user of any active running modes
+        if self.opts.devmode:
+            verbose('running in development mode')
+
+        if self.opts.local_srcs:
+            verbose('running in local-sources mode')
+
+        # register the project's root directory as a system path; permits a
+        # project to import locally created modules in their build/etc. scripts
+        debug('registering root directory in path...')
+        sys.path.append(opts.root_dir)
+
+        # register the project's host directory as a system path; lazily permits
+        # loading host tools (not following prefix or bin container) built by a
+        # project over the system
+        debug('registering host directory in path...')
+        sys.path.insert(0, opts.host_dir)
+        os.environ['PATH'] = opts.host_dir + os.pathsep + os.environ['PATH']
+
         # prepare script environment to make helpers available to configuration
         # script(s)
+        #
+        # Note that some options prepared for the configuration script may
+        # need to be updated after running the project's configuration (e.g.
+        # possibly needing to update the `PREFIX` variable if `sysroot_prefix`
+        # was configured). This is peformed later in the engine run call.
         self._prepare_script_environment(gbls, gaction, opts.pkg_action)
 
+        verbose("loading user's configuration...")
         conf_point, conf_point_exists = opt_file(opts.conf_point)
         if not conf_point_exists:
             raise RelengToolMissingConfigurationError(conf_point)
@@ -168,61 +195,65 @@ class RelengEngine:
             return False
         debug('target packages)')
         for pkg_name in pkg_names:
-            debug(' {}'.format(pkg_name))
+            debug(' {}', pkg_name)
 
         # processing additional settings
         if not self._process_settings(settings):
             return False
 
+        # register the project's host-bin directory as a system path; lazily
+        # permits loading host tools built by a project over the system
+        debug('registering host bin directory in path...')
+        host_bin_dir = os.path.join(opts.host_dir + opts.sysroot_prefix, 'bin')
+        sys.path.insert(0, host_bin_dir)
+        os.environ['PATH'] = host_bin_dir + os.pathsep + os.environ['PATH']
+
         # load and process packages
         pkgs = self.pkgman.load(pkg_names)
         if not pkgs:
             return False
 
+        # if cleaning a package, remove it's build output directory and stop
+        if pa in (PkgAction.CLEAN, PkgAction.DISTCLEAN):
+            for pkg in pkgs:
+                if pkg.name == opts.target_action:
+                    def pkg_verbose_clean(desc):
+                        verbose('{} for package: {}', desc, pkg.name)
+                    pkg_verbose_clean('removing output directory')
+                    path_remove(pkg.build_output_dir)
+
+                    if pa == PkgAction.DISTCLEAN:
+                        if os.path.exists(pkg.cache_file):
+                            pkg_verbose_clean('removing cache file')
+                            path_remove(pkg.cache_file)
+                        if os.path.isdir(pkg.cache_dir):
+                            pkg_verbose_clean('removing cache directory')
+                            path_remove(pkg.cache_dir)
+
+                    return True
+            assert False # should not reach here
+
+        # ensure any of required host tools do exist
+        if 'releng.disable_prerequisites_check' not in opts.quirks:
+            prerequisites = RelengPrerequisites(pkgs, opts.prerequisites)
+            if not prerequisites.check():
+                return False
+
+        # track if this action is "pre-configuration", where a package
+        # dependency chain can be ignored
+        requested_preconfig = pa in [
+            PkgAction.EXTRACT,
+            PkgAction.FETCH,
+            PkgAction.LICENSE,
+            PkgAction.PATCH,
+        ]
+
+        # determine if an explicit fetch request has been made
+        requested_fetch = None
+        if gaction == GlobalAction.FETCH or pa == PkgAction.FETCH:
+            requested_fetch = True
+
         try:
-            pa = opts.pkg_action
-            license_files = {}
-
-            # if cleaning a package, remove it's build output directory and stop
-            if pa in (PkgAction.CLEAN, PkgAction.DISTCLEAN):
-                for pkg in pkgs:
-                    if pkg.name == opts.target_action:
-                        def pkg_verbose_clean(desc):
-                            verbose('{} for package: {}', desc, pkg.name)
-                        pkg_verbose_clean('removing output directory')
-                        path_remove(pkg.build_output_dir)
-
-                        if pa == PkgAction.DISTCLEAN:
-                            if os.path.exists(pkg.cache_file):
-                                pkg_verbose_clean('removing cache file')
-                                path_remove(pkg.cache_file)
-                            if os.path.isdir(pkg.cache_dir):
-                                pkg_verbose_clean('removing cache directory')
-                                path_remove(pkg.cache_dir)
-
-                        return True
-                assert False # should not reach here
-
-            # ensure any of required host tools do exist
-            if 'releng.disable_prerequisites_check' not in opts.quirks:
-                prerequisites = RelengPrerequisites(pkgs, opts.prerequisites)
-                if not prerequisites.check():
-                    return False
-
-            # track if this action is "pre-configuration", where a package
-            # dependency chain can be ignored
-            requested_preconfig = pa in [
-                PkgAction.EXTRACT,
-                PkgAction.FETCH,
-                PkgAction.LICENSE,
-                PkgAction.PATCH,
-            ]
-
-            # determine if an explicit fetch request has been made
-            requested_fetch = None
-            if gaction == GlobalAction.FETCH or pa == PkgAction.FETCH:
-                requested_fetch = True
-
             # ensure all package sources are acquired first
             for pkg in pkgs:
                 if not self._stage_init(pkg):
@@ -237,23 +268,21 @@ class RelengEngine:
                 if pkg.vcs_type in (VcsType.LOCAL, VcsType.NONE):
                     continue
 
-                # in the event that we not not explicit fetching and the package
+                # in the event that we are not explicit fetching and the package
                 # has already been extracted, completely skip the fetching stage
-                if not requested_fetch:
+                is_local_pkg = pkg.is_internal and opts.local_srcs
+                if not requested_fetch and not is_local_pkg:
                     flag = pkg._ff_extract
                     if check_file_flag(flag) == FileFlag.EXISTS:
                         continue
 
                 self.stats.track_duration_start(pkg.name, 'fetch')
-                fetched = fetch_stage(self, pkg, ignore_cache=requested_fetch)
+                fetched = fetch_stage(
+                    self, pkg, requested_fetch, pkg.fetch_opts)
                 self.stats.track_duration_end(pkg.name, 'fetch')
                 if not fetched:
                     return False
 
-            # prepend project's host directory to path
-            host_bin = os.path.join(opts.host_dir, 'bin')
-            os.environ['PATH'] = host_bin + os.pathsep + os.environ['PATH']
-
             # re-apply script environment to ensure previous script environment
             # changes have not manipulated the environment (from standard
             # helpers).
@@ -263,11 +292,25 @@ class RelengEngine:
             # process each package (configuring, building, etc.)
             if gaction != GlobalAction.FETCH and pa != PkgAction.FETCH:
 
-                # ensure the symbols directory exists, as a package may wish to
-                # populate it anytime between a configuration stage to a
-                # post-package stage
-                if not ensure_dir_exists(opts.symbols_dir):
-                    return False
+                # pre-populate "common" directories
+                #
+                # The following will attempt to pre-populate common directories
+                # before attempting to process a package pipeline. A package
+                # configured in this pipeline may wish to populate any one of
+                # these directories anytime between a configuration stage to a
+                # post-package stage. Ensuring these directories exist ahead of
+                # time can help prevent scenarios where developers try to file
+                # a file into a directory, but instead create a file as the
+                # common directory name (e.g. `TARGET_DIR` points to a file).
+                common_dirs = [
+                    opts.host_dir,
+                    opts.staging_dir,
+                    opts.symbols_dir,
+                    opts.target_dir,
+                ]
+                for common_dir in common_dirs:
+                    if not ensure_dir_exists(common_dir):
+                        return False
 
                 pipeline = RelengPackagePipeline(self, opts, script_env)
                 for pkg in pkgs:
@@ -276,20 +319,21 @@ class RelengEngine:
                     if requested_preconfig and pkg.name != opts.target_action:
                         continue
 
-                    verbose('processing package: {}'.format(pkg.name))
+                    verbose('processing package: {}', pkg.name)
                     if not pipeline.process(pkg):
-                        break
+                        return True
                 license_files = pipeline.license_files
 
         except RelengToolStageFailure:
             return False
         except FailedToPrepareWorkingDirectoryError as e:
-            err("unable to prepare a package's working directory")
-            err("""\
+            err('''\
+unable to prepare a package's working directory
+
 An attempt to prepare and move into a working directory for a package process
 has failed. Ensure the following path is accessible for this user:
 
-    {}""".format(e))
+    {}''', e)
             return False
 
         is_action = (gaction or pa or opts.target_action is not None)
@@ -309,8 +353,8 @@ has failed. Ensure the following path is accessible for this user:
                 err('failed to perform post-processing')
                 return False
 
-            self.end_time = datetime.now().replace(microsecond=0)
-            success('completed ({})'.format(self.end_time - self.start_time))
+            end_time = datetime.now().replace(microsecond=0)
+            success('completed ({})', end_time - start_time)
 
         # attempt to generate a report at the end of a run
         self.stats.generate()
@@ -332,12 +376,23 @@ has failed. Ensure the following path is accessible for this user:
             otherwise
         """
 
+        # forced request
+        #
+        # When a forced request is made, clear all file flags so these stages
+        # can be invoked again.
+        if self.opts.force:
+            path_remove(pkg._ff_bootstrap)
+            path_remove(pkg._ff_configure)
+            path_remove(pkg._ff_build)
+            path_remove(pkg._ff_install)
+            path_remove(pkg._ff_post)
+
         # user invoking a package-specific override
         #
         # If the user is invoking a package rebuild, reconfiguration, etc.,
         # ensure existing file flags are cleared to ensure these stages are
         # invoked again.
-        if pkg.name == self.opts.target_action:
+        elif pkg.name == self.opts.target_action:
             if (self.opts.pkg_action in
                     (PkgAction.REBUILD, PkgAction.REBUILD_ONLY)):
                 path_remove(pkg._ff_build)
@@ -361,7 +416,7 @@ has failed. Ensure the following path is accessible for this user:
         acquire list of project package names to process
 
         From a dictionary of user-defined settings, extract the known list of
-        package names from the package configuration key ``CONF_KEY_PKGS``. This
+        package names from the package configuration key ``ConfKey.PKGS``. This
         method will return a (duplicate-removed) list of packages (if any) to be
         processed.
 
@@ -376,8 +431,8 @@ has failed. Ensure the following path is accessible for this user:
         pkg_names = []
         bad_pkgs_value = False
 
-        if CONF_KEY_PKGS in settings:
-            pkg_names = interpret_strings(settings[CONF_KEY_PKGS])
+        if ConfKey.PKGS in settings:
+            pkg_names = interpret_strings(settings[ConfKey.PKGS])
             if pkg_names is None:
                 bad_pkgs_value = True
 
@@ -385,18 +440,20 @@ has failed. Ensure the following path is accessible for this user:
         pkg_names = OrderedDict.fromkeys(pkg_names)
 
         if bad_pkgs_value:
-            err('bad package list definition')
-            err("""\
+            err('''\
+bad package list definition
+
 The configuration file does not have a properly formed list of defined packages.
 Ensure a package list exists with the string-based names of packages to be part
 of the releng process:
 
     {}
-        {} = ['liba', 'libb', 'libc']""".format(
-                self.opts.conf_point, CONF_KEY_PKGS))
+        {} = ['liba', 'libb', 'libc']''',
+                self.opts.conf_point,
+                ConfKey.PKGS)
         elif not pkg_names:
             raise RelengToolMissingPackagesError(
-                self.opts.conf_point, CONF_KEY_PKGS)
+                self.opts.conf_point, ConfKey.PKGS)
 
         return pkg_names
 
@@ -473,29 +530,29 @@ of the releng process:
                     license_header = 'license(s)'
 
                 # output license header
-                dst.write("""{}
+                dst.write('''{}
 ################################################################################
-""".format(license_header))
+'''.format(license_header))
 
                 # output license header
                 for license_name, license_data in sorted(license_files.items()):
                     license_files = license_data['files']
                     license_version = license_data['version']
-                    dst.write("""
+                    dst.write('''
 {}-{}
 --------------------------------------------------------------------------------
-""".format(license_name, license_version))
+'''.format(license_name, license_version))
                     for pkg_license_file in sorted(license_files):
-                        verbose('writing license file ({}): {}'.format(
-                            license_name, pkg_license_file))
+                        verbose('writing license file ({}): {}',
+                            license_name, pkg_license_file)
                         with open(pkg_license_file, 'r') as f:
                             copyfileobj(f, dst)
                         dst.write('')
 
             verbose('license file has been written')
         except IOError as e:
-            err('unable to populate license information')
-            err('    {}'.format(e))
+            err('unable to populate license information\n'
+                '    {}', e)
             return False
 
         return True
@@ -569,6 +626,7 @@ of the releng process:
         script_env['RELENG_DEBUG'] = None
         script_env['RELENG_DEVMODE'] = None
         script_env['RELENG_DISTCLEAN'] = None
+        script_env['RELENG_FORCE'] = None
         script_env['RELENG_LOCALSRCS'] = None
         script_env['RELENG_MRPROPER'] = None
         script_env['RELENG_REBUILD'] = None
@@ -576,23 +634,44 @@ of the releng process:
         script_env['RELENG_REINSTALL'] = None
         script_env['RELENG_VERBOSE'] = None
 
+        #: default lib container directory
+        sysroot_nprefix = os.path.normpath(self.opts.sysroot_prefix)
+        host_pdir = self.opts.host_dir + sysroot_nprefix
+        staging_pdir = self.opts.staging_dir + sysroot_nprefix
+        target_pdir = self.opts.target_dir + sysroot_nprefix
+        host_include_dir = os.path.join(host_pdir, 'include')
+        host_lib_dir = os.path.join(host_pdir, 'lib')
+        staging_include_dir = os.path.join(staging_pdir, 'include')
+        staging_lib_dir = os.path.join(staging_pdir, 'lib')
+        target_include_dir = os.path.join(target_pdir, 'include')
+        target_lib_dir = os.path.join(target_pdir, 'lib')
+
         # global variables
         for env in (os.environ, script_env):
             env['BUILD_DIR'] = self.opts.build_dir
             env['CACHE_DIR'] = self.opts.cache_dir
             env['DL_DIR'] = self.opts.dl_dir
             env['HOST_DIR'] = self.opts.host_dir
+            env['HOST_INCLUDE_DIR'] = host_include_dir
+            env['HOST_LIB_DIR'] = host_lib_dir
             env['IMAGES_DIR'] = self.opts.images_dir
             env['LICENSE_DIR'] = self.opts.license_dir
             env['NJOBS'] = str(self.opts.jobs)
             env['NJOBSCONF'] = str(self.opts.jobsconf)
             env['OUTPUT_DIR'] = self.opts.out_dir
             env['PREFIX'] = self.opts.sysroot_prefix
+            env['PREFIXED_HOST_DIR'] = host_pdir
+            env['PREFIXED_STAGING_DIR'] = staging_pdir
+            env['PREFIXED_TARGET_DIR'] = target_pdir
             env['RELENG_VERSION'] = releng_version
             env['ROOT_DIR'] = self.opts.root_dir
             env['STAGING_DIR'] = self.opts.staging_dir
+            env['STAGING_INCLUDE_DIR'] = staging_include_dir
+            env['STAGING_LIB_DIR'] = staging_lib_dir
             env['SYMBOLS_DIR'] = self.opts.symbols_dir
             env['TARGET_DIR'] = self.opts.target_dir
+            env['TARGET_INCLUDE_DIR'] = target_include_dir
+            env['TARGET_LIB_DIR'] = target_lib_dir
 
             if gaction == GlobalAction.CLEAN:
                 env['RELENG_CLEAN'] = '1'
@@ -615,6 +694,8 @@ of the releng process:
                 env['RELENG_DEBUG'] = '1'
             if self.opts.devmode:
                 env['RELENG_DEVMODE'] = '1'
+            if self.opts.force:
+                env['RELENG_FORCE'] = '1'
             if self.opts.local_srcs:
                 env['RELENG_LOCALSRCS'] = '1'
             if self.opts.verbose:
@@ -625,6 +706,7 @@ of the releng process:
         script_env['err'] = err
         script_env['log'] = log
         script_env['note'] = note
+        script_env['releng_cat'] = cat
         script_env['releng_copy'] = path_copy
         script_env['releng_env'] = env_value
         script_env['releng_execute'] = execute
@@ -632,10 +714,13 @@ of the releng process:
         script_env['releng_exists'] = path_exists
         script_env['releng_exit'] = platform_exit
         script_env['releng_expand'] = expand
+        script_env['releng_include'] = releng_include
         script_env['releng_join'] = os.path.join
+        script_env['releng_ls'] = ls
         script_env['releng_mkdir'] = ensure_dir_exists
         script_env['releng_move'] = path_move
         script_env['releng_remove'] = path_remove
+        script_env['releng_require_version'] = require_version
         script_env['releng_tmpdir'] = generate_temp_dir
         script_env['releng_touch'] = touch
         script_env['releng_wd'] = interim_working_dir
@@ -706,102 +791,112 @@ of the releng process:
             configure file flags failed to be performed
         """
         def notify_invalid_value(key, expected):
-            err('invalid configuration value provided')
-            err("""\
+            err('''\
+invalid configuration value provided
+
 The configuration file defines a key with an unexpected type. Correct the
 following key entry and re-try again.
 
     Key: {}
-    Expected Type: {}""".format(key, expected))
+    Expected Type: {}''', key, expected)
 
-        if CONF_KEY_CACHE_EXT_TRANSFORM in settings:
+        if ConfKey.CACHE_EXT_TRANSFORM in settings:
             cet = None
-            if callable(settings[CONF_KEY_CACHE_EXT_TRANSFORM]):
-                cet = settings[CONF_KEY_CACHE_EXT_TRANSFORM]
+            if callable(settings[ConfKey.CACHE_EXT_TRANSFORM]):
+                cet = settings[ConfKey.CACHE_EXT_TRANSFORM]
             if cet is None:
-                notify_invalid_value(CONF_KEY_CACHE_EXT_TRANSFORM, 'callable')
+                notify_invalid_value(ConfKey.CACHE_EXT_TRANSFORM, 'callable')
                 return False
             self.opts.cache_ext_transform = cet
 
-        if CONF_KEY_DEFINTERN in settings:
-            is_default_internal = settings[CONF_KEY_DEFINTERN]
+        if ConfKey.DEFINTERN in settings:
+            is_default_internal = settings[ConfKey.DEFINTERN]
             if not isinstance(is_default_internal, bool):
-                notify_invalid_value(CONF_KEY_DEFINTERN, 'bool')
+                notify_invalid_value(ConfKey.DEFINTERN, 'bool')
                 return False
             self.opts.default_internal_pkgs = is_default_internal
 
-        if CONF_KEY_LICENSE_HEADER in settings:
-            license_header = interpret_string(settings[CONF_KEY_LICENSE_HEADER])
+        if ConfKey.LICENSE_HEADER in settings:
+            license_header = interpret_string(settings[ConfKey.LICENSE_HEADER])
             if license_header is None:
-                notify_invalid_value(CONF_KEY_LICENSE_HEADER, 'str')
+                notify_invalid_value(ConfKey.LICENSE_HEADER, 'str')
                 return False
             self.opts.license_header = license_header
 
-        if CONF_KEY_OVERRIDE_REV in settings:
-            orz = interpret_dictionary_strings(settings[CONF_KEY_OVERRIDE_REV])
+        if ConfKey.OVERRIDE_REV in settings:
+            orz = interpret_dictionary_strings(settings[ConfKey.OVERRIDE_REV])
             if orz is None:
-                notify_invalid_value(CONF_KEY_OVERRIDE_REV, 'dict(str,str)')
+                notify_invalid_value(ConfKey.OVERRIDE_REV, 'dict(str,str)')
                 return False
             self.opts.revision_override = orz
 
-        if CONF_KEY_OVERRIDE_SITES in settings:
-            osz = interpret_dictionary_strings(settings[CONF_KEY_OVERRIDE_SITES])
-            if osz is None:
-                notify_invalid_value(CONF_KEY_OVERRIDE_SITES, 'dict(str,str)')
+        if ConfKey.OVERRIDE_SITES in settings:
+            v = interpret_dictionary_strings(settings[ConfKey.OVERRIDE_SITES])
+            if v is None:
+                notify_invalid_value(ConfKey.OVERRIDE_SITES, 'dict(str,str)')
                 return False
-            self.opts.sites_override = osz
+            self.opts.sites_override = v
 
-        if CONF_KEY_OVERRIDE_TOOLS in settings:
-            otz = interpret_dictionary_strings(settings[CONF_KEY_OVERRIDE_TOOLS])
-            if otz is None:
-                notify_invalid_value(CONF_KEY_OVERRIDE_TOOLS, 'dict(str,str)')
+        if ConfKey.OVERRIDE_TOOLS in settings:
+            v = interpret_dictionary_strings(settings[ConfKey.OVERRIDE_TOOLS])
+            if v is None:
+                notify_invalid_value(ConfKey.OVERRIDE_TOOLS, 'dict(str,str)')
                 return False
-            self.opts.extract_override = otz
+            self.opts.extract_override = v
 
-        if CONF_KEY_PREREQUISITES in settings:
-            prerequisites = interpret_strings(settings[CONF_KEY_PREREQUISITES])
+        if ConfKey.PREREQUISITES in settings:
+            prerequisites = interpret_strings(settings[ConfKey.PREREQUISITES])
             if prerequisites is None:
-                notify_invalid_value(CONF_KEY_PREREQUISITES, 'str or list(str)')
+                notify_invalid_value(ConfKey.PREREQUISITES, 'str or list(str)')
                 return False
             self.opts.prerequisites.extend(prerequisites)
 
-        if CONF_KEY_QUIRKS in settings:
-            quirks = interpret_strings(settings[CONF_KEY_QUIRKS])
+        if ConfKey.QUIRKS in settings:
+            quirks = interpret_strings(settings[ConfKey.QUIRKS])
             if quirks is None:
-                notify_invalid_value(CONF_KEY_QUIRKS, 'str or list(str)')
+                notify_invalid_value(ConfKey.QUIRKS, 'str or list(str)')
                 return False
             self.opts.quirks.extend(quirks)
             for quirk in quirks:
                 verbose('configuration quirk applied: ' + quirk)
 
-        if CONF_KEY_SYSROOT_PREFIX in settings:
-            sysroot_prefix = interpret_string(settings[CONF_KEY_SYSROOT_PREFIX])
+        if ConfKey.SYSROOT_PREFIX in settings:
+            sysroot_prefix = interpret_string(settings[ConfKey.SYSROOT_PREFIX])
             if sysroot_prefix is None:
-                notify_invalid_value(CONF_KEY_SYSROOT_PREFIX, 'str')
+                notify_invalid_value(ConfKey.SYSROOT_PREFIX, 'str')
                 return False
             if not sysroot_prefix.startswith('/'):
                 sysroot_prefix = '/' + sysroot_prefix
             self.opts.sysroot_prefix = sysroot_prefix
 
-        if CONF_KEY_URL_MIRROR in settings:
-            url_mirror = interpret_string(settings[CONF_KEY_URL_MIRROR])
+        if ConfKey.URL_MIRROR in settings:
+            url_mirror = interpret_string(settings[ConfKey.URL_MIRROR])
             if url_mirror is None:
-                notify_invalid_value(CONF_KEY_URL_MIRROR, 'str')
+                notify_invalid_value(ConfKey.URL_MIRROR, 'str')
                 return False
             self.opts.url_mirror = url_mirror
 
-        if CONF_KEY_EXTEN_PKGS in settings:
-            epd = interpret_strings(settings[CONF_KEY_EXTEN_PKGS])
+        if ConfKey.URLOPEN_CONTEXT in settings:
+            urlopen_context = None
+            if isinstance(settings[ConfKey.URLOPEN_CONTEXT], ssl.SSLContext):
+                urlopen_context = settings[ConfKey.URLOPEN_CONTEXT]
+            if urlopen_context is None:
+                notify_invalid_value(ConfKey.URLOPEN_CONTEXT, 'ssl.SSLContext')
+                return False
+            self.opts.urlopen_context = urlopen_context
+
+        if ConfKey.EXTEN_PKGS in settings:
+            epd = interpret_strings(settings[ConfKey.EXTEN_PKGS])
             if epd is None:
-                notify_invalid_value(CONF_KEY_EXTEN_PKGS, 'str or list(str)')
+                notify_invalid_value(ConfKey.EXTEN_PKGS, 'str or list(str)')
                 return False
             self.opts.extern_pkg_dirs = epd
 
         ext_names = []
-        if CONF_KEY_EXTENSIONS in settings:
-            ext_names = interpret_strings(settings[CONF_KEY_EXTENSIONS])
+        if ConfKey.EXTENSIONS in settings:
+            ext_names = interpret_strings(settings[ConfKey.EXTENSIONS])
             if ext_names is None:
-                notify_invalid_value(CONF_KEY_EXTENSIONS, 'str or list(str)')
+                notify_invalid_value(ConfKey.EXTENSIONS, 'str or list(str)')
                 return False
 
         self.registry.load_all_extensions(ext_names)
