@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2022 releng-tool
 
 from releng_tool.api import RelengFetchOptions
 from releng_tool.defs import VcsType
@@ -7,6 +7,7 @@ from releng_tool.fetch.bzr import fetch as fetch_bzr
 from releng_tool.fetch.cvs import fetch as fetch_cvs
 from releng_tool.fetch.git import fetch as fetch_git
 from releng_tool.fetch.mercurial import fetch as fetch_mercurial
+from releng_tool.fetch.rsync import fetch as fetch_rsync
 from releng_tool.fetch.scp import fetch as fetch_scp
 from releng_tool.fetch.svn import fetch as fetch_svn
 from releng_tool.fetch.url import fetch as fetch_url
@@ -25,7 +26,7 @@ from releng_tool.util.log import warn
 import os
 import shutil
 
-def stage(engine, pkg, ignore_cache):
+def stage(engine, pkg, ignore_cache, extra_opts):
     """
     handles the fetching stage for a package
 
@@ -36,6 +37,7 @@ def stage(engine, pkg, ignore_cache):
         engine: the engine
         pkg: the package being fetched
         ignore_cache: always attempt to ignore the cache
+        extra_opts: extra options for the fetch operation (if applicable)
 
     Returns:
         ``True`` if the fetching stage is completed; ``False`` otherwise
@@ -49,15 +51,16 @@ def stage(engine, pkg, ignore_cache):
         if os.path.isdir(pkg.build_dir):
             return True
 
-        err('missing local sources for internal package: ' + name)
-        err("""\
+        err('''\
+missing local sources for internal package: {0}
+
 The active configuration is flagged for 'local sources' mode; however, an
 internal package cannot be found in the local system. Before continuing, ensure
 you have checked out all internal packages on your local system (or, disable the
 local sources option to use the default process).
 
-       Package: {}
- Expected Path: {}""".format(name, pkg.build_dir))
+       Package: {0}
+ Expected Path: {1}''', name, pkg.build_dir)
         return False
 
     # if the vcs-type is archive-based, flag that hash checks are needed
@@ -71,12 +74,15 @@ local sources option to use the default process).
     replicate_package_attribs(fetch_opts, pkg)
     fetch_opts.cache_dir = pkg.cache_dir
     fetch_opts.ext = pkg.ext_modifiers
+    fetch_opts.extra_opts = extra_opts
     fetch_opts.ignore_cache = ignore_cache
     fetch_opts.name = name
     fetch_opts.revision = pkg.revision
     fetch_opts.site = pkg.site
     fetch_opts.version = pkg.version
+    fetch_opts._mirror = False
     fetch_opts._quirks = engine.opts.quirks
+    fetch_opts._urlopen_context = engine.opts.urlopen_context
 
     cache_filename = os.path.basename(pkg.cache_file)
     out_dir = engine.opts.out_dir
@@ -105,7 +111,15 @@ local sources option to use the default process).
                     verbose('removing cache file (per configuration): ' + name)
                     if not path_remove(pkg.cache_file):
                         return False
-            # force explicit ignore cache when not in development mode
+
+            # remove cache file if there is a force request to ignore the cache
+            elif engine.opts.force and ignore_cache:
+                if os.path.exists(pkg.cache_file):
+                    verbose('removing cache file (forced): ' + name)
+                    if not path_remove(pkg.cache_file):
+                        return False
+
+            # force explicit ignore cache (to off) when not in development mode
             elif not engine.opts.devmode and ignore_cache is None:
                 fetch_opts.ignore_cache = False
 
@@ -132,18 +146,18 @@ local sources option to use the default process).
                         rv = False
                     elif hr == HashResult.MISSING_ARCHIVE:
                         if not perform_file_asc_check:
-                            err("""\
+                            err('''\
 missing archive hash for verification
 
 The hash file for this package does not have an entry for the cache file to be
 verified. Ensure the hash file defines an entry for the expected cache file:
 
     Hash File: {}
-         File: {}""".format(pkg.hash_file, cache_filename))
+         File: {}''', pkg.hash_file, cache_filename)
                             rv = False
                     else:
                         err('invalid fetch operation (internal error; '
-                            'hash-check failure: {})'.format(hr))
+                            'hash-check failure: {})', hr)
                         rv = False
                 else:
                     rv = True
@@ -154,14 +168,14 @@ verified. Ensure the hash file defines an entry for the expected cache file:
                         rv = True
                     else:
                         if not path_remove(pkg.cache_file):
-                            err("""\
+                            err('''\
 failed to validate against ascii-armor
 
 Validation of a package resource failed to verify against a provided ASCII-armor
 file. Ensure that the package's public key has been registered into gpg.
 
  ASC File: {}
-     File: {}""".format(pkg.asc_file, cache_filename))
+     File: {}''', pkg.asc_file, cache_filename)
                             rv = False
                         else:
                             rv = None
@@ -187,6 +201,8 @@ file. Ensure that the package's public key has been registered into gpg.
                 fetcher = fetch_git
             elif pkg.vcs_type == VcsType.HG:
                 fetcher = fetch_mercurial
+            elif pkg.vcs_type == VcsType.RSYNC:
+                fetcher = fetch_rsync
             elif pkg.vcs_type == VcsType.SCP:
                 fetcher = fetch_scp
             elif pkg.vcs_type == VcsType.SVN:
@@ -195,7 +211,7 @@ file. Ensure that the package's public key has been registered into gpg.
                 fetcher = fetch_url
 
             if not fetcher:
-                err('fetch type is not implemented: {}'.format(pkg.vcs_type))
+                err('fetch type is not implemented: {}', pkg.vcs_type)
                 return False
 
             # if this is url-type location, attempt to search on the mirror
@@ -205,10 +221,14 @@ file. Ensure that the package's public key has been registered into gpg.
                 original_site = fetch_opts.site
                 new_site = engine.opts.url_mirror + cache_filename
                 if original_site != new_site:
+                    fetch_opts._mirror = True
+
                     fetch_opts.site = new_site
                     fetched = fetcher(fetch_opts)
                     fetch_opts.site = original_site
 
+                    fetch_opts._mirror = False
+
             # perform the fetch request (if not already fetched)
             if not fetched:
                 fetched = fetcher(fetch_opts)
@@ -238,30 +258,30 @@ file. Ensure that the package's public key has been registered into gpg.
                         return False
                     elif hr == HashResult.MISSING_ARCHIVE:
                         if not perform_file_asc_check:
-                            err("""\
+                            err('''\
 missing archive hash for verification
 
 The hash file for this package does not have an entry for the cache file to be
 verified. Ensure the hash file defines an entry for the expected cache file:
 
     Hash File: {}
-         File: {}""".format(pkg.hash_file, cache_filename))
+         File: {}''', pkg.hash_file, cache_filename)
                             return False
                     else:
                         err('invalid fetch operation (internal error; '
-                            'hash-check failure: {})'.format(hr))
+                            'hash-check failure: {})', hr)
                         return False
 
                 if perform_file_asc_check:
                     if not GPG.validate(pkg.asc_file, interim_cache_file):
-                        err("""\
+                        err('''\
 failed to validate against ascii-armor
 
 Validation of a package resource failed to verify against a provided ASCII-armor
 file. Ensure that the package's public key has been registered into gpg.
 
      ASC File: {}
-         File: {}""".format(pkg.asc_file, cache_filename))
+         File: {}''', pkg.asc_file, cache_filename)
                         return False
 
                 debug('fetch successful; moving cache file')
@@ -274,12 +294,11 @@ file. Ensure that the package's public key has been registered into gpg.
                     shutil.move(interim_cache_file, pkg.cache_file)
                 except shutil.Error:
                     err('invalid fetch operation (internal error; fetch mode '
-                        '"{}" has provided a missing cache file)'.format(
-                            pkg.vcs_type))
+                        '"{}" has provided a missing cache file)', pkg.vcs_type)
                     return False
             else:
                 err('invalid fetch operation (internal error; fetch mode "{}" '
-                    'has returned an unsupported value)'.format(pkg.vcs_type))
+                    'has returned an unsupported value)', pkg.vcs_type)
                 return False
 
     return True
