@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2022 releng-tool
 
 from releng_tool.packages import pkg_cache_key
 from releng_tool.tool.git import GIT
@@ -84,7 +84,7 @@ def fetch(opts):
                     git_dir, revision, quiet=True):
                 return cache_dir
 
-    note('fetching {}...'.format(name))
+    note('fetching {}...', name)
     sys.stdout.flush()
 
     # validate any cache directory (if one exists)
@@ -97,8 +97,7 @@ def fetch(opts):
         if not ensure_dir_exists(cache_dir):
             return None
 
-        if not GIT.execute([git_dir, 'init', '--bare'], cwd=cache_dir):
-            err('unable to initialize bare git repository')
+        if not _create_bare_git_repo(cache_dir):
             return None
 
     # ensure configuration is properly synchronized
@@ -112,7 +111,7 @@ def fetch(opts):
     # verify revision (if configured to check it)
     if opts._git_verify_revision:
         if not _verify_revision(git_dir, revision):
-            err("""\
+            err('''\
 failed to validate git revision
 
 Package has been configured to require the verification of the GPG signature
@@ -120,7 +119,7 @@ for the target revision. The verification has failed. Ensure that the revision
 is signed and that the package's public key has been registered in the system.
 
       Package: {}
-     Revision: {}""".format(name, revision))
+     Revision: {}''', name, revision)
             return None
 
     # fetch submodules (if configured to do so)
@@ -184,10 +183,11 @@ def _fetch_srcs(opts, cache_dir, revision, desc=None, refspecs=None):
     if revision and quick_fetch:
         ls_cmd = [
             'ls-remote',
+            '--exit-code',
             'origin',
         ]
         debug('checking if tag exists on remote')
-        if GIT.execute(ls_cmd + ['--tags', revision],
+        if GIT.execute(ls_cmd + ['--tags', 'refs/tags/{}'.format(revision)],
                 cwd=cache_dir, quiet=True):
             debug('attempting a tag reference fetch operation')
             fetch_cmd = list(prepared_fetch_cmd)
@@ -200,7 +200,7 @@ def _fetch_srcs(opts, cache_dir, revision, desc=None, refspecs=None):
                 return True
 
         debug('checking if reference exists on remote')
-        if GIT.execute(ls_cmd + ['--heads', revision],
+        if GIT.execute(ls_cmd + ['--heads', 'refs/heads/{}'.format(revision)],
                 cwd=cache_dir, quiet=True):
             debug('attempting a head reference fetch operation')
             fetch_cmd = list(prepared_fetch_cmd)
@@ -251,12 +251,12 @@ def _fetch_srcs(opts, cache_dir, revision, desc=None, refspecs=None):
                 return False
 
             if revision_exists(git_dir, revision) not in REVISION_EXISTS:
-                err('unable to find matching revision in ' + desc)
-                err(' (revision: {}) '.format(revision))
+                err('unable to find matching revision in {}\n'
+                    ' (revision: {})', desc, revision)
                 return False
         else:
-            err('unable to find matching revision in ' + desc)
-            err(' (revision: {}) '.format(revision))
+            err('unable to find matching revision in {}\n'
+                'revision: {})', desc, revision)
             return False
 
     return True
@@ -302,6 +302,29 @@ def revision_exists(git_dir, revision):
 
     return GitExistsType.EXISTS_BRANCH
 
+def _create_bare_git_repo(cache_dir):
+    """
+    create a bare git repository
+
+    This call will build a bare Git repository in the provided cache
+    directory. If the repository could not be created, an error message
+    will be generated and this method will return ``False``.
+
+    Args:
+        cache_dir: the cache/bare repository
+
+    Returns:
+        ``True`` if the repository could be created; ``False`` otherwise
+    """
+
+    git_dir = '--git-dir=' + cache_dir
+
+    if GIT.execute([git_dir, 'init', '--bare', '--quiet'], cwd=cache_dir):
+        return True
+
+    err('unable to initialize bare git repository')
+    return False
+
 def _sync_git_configuration(opts):
     """
     ensure the git configuration is properly synchronized with this repository
@@ -488,7 +511,7 @@ def _fetch_submodule(opts, name, cache_dir, revision, site):
         if revision_exists(git_dir, revision) in REVISION_EXISTS:
             return _sync_git_origin(cache_dir, site)
 
-    log('processing submodule (package: {}) {}...'.format(opts.name, name))
+    log('processing submodule (package: {}) {}...', opts.name, name)
     sys.stdout.flush()
 
     # validate any cache directory (if one exists)
@@ -501,8 +524,7 @@ def _fetch_submodule(opts, name, cache_dir, revision, site):
         if not ensure_dir_exists(cache_dir):
             return False
 
-        if not GIT.execute([git_dir, 'init', '--bare'], cwd=cache_dir):
-            err('unable to initialize bare git repository')
+        if not _create_bare_git_repo(cache_dir):
             return False
 
     # ensure configuration is properly synchronized
