@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2022 releng-tool
 
 from releng_tool.defs import GlobalAction
 from releng_tool.defs import PkgAction
@@ -44,6 +44,7 @@ class RelengEngineOptions:
             be forwarded to the releng-tool project's configuration
 
     Attributes:
+        assets_dir: directory container for cache/download directories
         build_dir: directory container for all builds
         cache_dir: directory container for cache (vcs bare sources)
         cache_ext_transform: transform for cache extension from site path
@@ -58,10 +59,12 @@ class RelengEngineOptions:
         extract_override: dictionary to override extraction commands
         ff_devmode: the file flag path for development mode detection
         ff_local_srcs: the file flag path for local sources mode detection
+        force: whether or not the force flag is set
         forward_args: command line arguments forwarded to configuration script
         gbl_action: the specific global-action to perform (if any)
         host_dir: directory container for host tools
         images_dir: directory container for (final) images
+        injected_kv: injected key-value entries for script/env context
         jobs: number of calculated jobs to allow at a given time
         jobsconf: number of jobs to allow at a given time (0: auto)
         license_dir: directory container for license information
@@ -80,11 +83,14 @@ class RelengEngineOptions:
         symbols_dir: directory container for symbols content
         sysroot_prefix: system root prefix
         target_action: the specific package to work on (if any)
+        target_action_exec: package-specific executable command (if any)
         target_dir: directory container for target content
         url_mirror: mirror base site for url fetches
+        urlopen_context: context to apply for all url open calls
         verbose: whether or not verbose messages are shown
     """
     def __init__(self, args=None, forward_args=None):
+        self.assets_dir = None
         self.build_dir = None
         self.cache_dir = None
         self.cache_ext_transform = None
@@ -99,10 +105,12 @@ class RelengEngineOptions:
         self.extract_override = None
         self.ff_devmode = None
         self.ff_local_srcs = None
+        self.force = False
         self.forward_args = forward_args
         self.gbl_action = None
         self.host_dir = None
         self.images_dir = None
+        self.injected_kv = {}
         self.jobs = 1
         self.jobsconf = 0
         self.license_dir = None
@@ -121,12 +129,15 @@ class RelengEngineOptions:
         self.symbols_dir = None
         self.sysroot_prefix = DEFAULT_SYSROOT_PREFIX
         self.target_action = None
+        self.target_action_exec = None
         self.target_dir = None
         self.url_mirror = None
+        self.urlopen_context = None
         self.verbose = False
 
         if args:
             self._handle_arguments(args)
+        self._handle_environment_opts()
         self._finalize_options()
 
     def _handle_arguments(self, args):
@@ -139,6 +150,8 @@ class RelengEngineOptions:
         Args:
             args: the arguments
         """
+        if args.assets_dir:
+            self.assets_dir = os.path.abspath(args.assets_dir)
         if args.cache_dir:
             self.cache_dir = os.path.abspath(args.cache_dir)
         if args.dl_dir:
@@ -149,12 +162,15 @@ class RelengEngineOptions:
             self.root_dir = os.path.abspath(args.root_dir)
 
         self.debug = args.debug
+        self.force = args.force
         self.jobs = self.jobsconf = (args.jobs or 0)
         self.no_color_out = args.nocolorout
         self.verbose = args.verbose
 
         if args.development:
             self.devmode = True
+        if args.injected_kv:
+            self.injected_kv = args.injected_kv
         if args.local_sources:
             self.local_srcs = True
         if args.quirk:
@@ -172,9 +188,26 @@ class RelengEngineOptions:
                         self.target_action = args.action[:idx]
                         break
 
-                if not self.pkg_action:
+                if subaction_val == PkgAction.EXEC:
+                    self.target_action_exec = args.action_exec
+                elif not self.pkg_action:
                     self.target_action = args.action
 
+    def _handle_environment_opts(self):
+        """
+        handle environment variables to populate engine options
+
+        Configure various options which support assignment from an
+        environment variable.
+        """
+
+        if not self.assets_dir:
+            self.assets_dir = os.environ.get('RELENG_ASSETS_DIR')
+        if not self.cache_dir:
+            self.cache_dir = os.environ.get('RELENG_CACHE_DIR')
+        if not self.dl_dir:
+            self.dl_dir = os.environ.get('RELENG_DL_DIR')
+
     def _finalize_options(self):
         """
         finalize all engine options for use
@@ -189,6 +222,12 @@ class RelengEngineOptions:
         join = os.path.join
         root = self.root_dir
 
+        # asset container
+        if self.assets_dir:
+            if not self.cache_dir:
+                self.cache_dir = join(self.assets_dir, DEFAULT_CACHE_DIR)
+            if not self.dl_dir:
+                self.dl_dir = join(self.assets_dir, DEFAULT_DL_DIR)
         # root container
         if not self.cache_dir:
             self.cache_dir = join(root, DEFAULT_CACHE_DIR)
@@ -236,4 +275,14 @@ class RelengEngineOptions:
         # non-zero, or the value will be a non-zero value matching the CPU count
         # of the running system.
         if not self.jobs:
-            self.jobs = multiprocessing.cpu_count()
+            try:
+                # if `sched_getaffinity` is available, use the call the acquire
+                # the number of physical cores on the system
+                self.jobs = len(os.sched_getaffinity(0))
+            except AttributeError:
+                # if we cannot guarantee the number of physical cores, make an
+                # assumption that the physical core count is half of the
+                # (possible logical) core count provided by `cpu_count`
+                cpu_count = multiprocessing.cpu_count()
+                fuzzy_phy_cores = max(int(cpu_count / 2), 1)
+                self.jobs = fuzzy_phy_cores
