@@ -1,47 +1,9 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2022 releng-tool
 
 from releng_tool.defs import PackageInstallType
 from releng_tool.defs import PackageType
-from releng_tool.defs import RPK_AUTOTOOLS_AUTORECONF
-from releng_tool.defs import RPK_BUILD_DEFS
-from releng_tool.defs import RPK_BUILD_ENV
-from releng_tool.defs import RPK_BUILD_OPTS
-from releng_tool.defs import RPK_BUILD_SUBDIR
-from releng_tool.defs import RPK_CONF_DEFS
-from releng_tool.defs import RPK_CONF_ENV
-from releng_tool.defs import RPK_CONF_OPTS
-from releng_tool.defs import RPK_DEPS
-from releng_tool.defs import RPK_DEVMODE_IGNORE_CACHE
-from releng_tool.defs import RPK_DEVMODE_REVISION
-from releng_tool.defs import RPK_EXTENSION
-from releng_tool.defs import RPK_EXTERNAL
-from releng_tool.defs import RPK_EXTOPT
-from releng_tool.defs import RPK_EXTRACT_TYPE
-from releng_tool.defs import RPK_FIXED_JOBS
-from releng_tool.defs import RPK_GIT_CONFIG
-from releng_tool.defs import RPK_GIT_DEPTH
-from releng_tool.defs import RPK_GIT_REFSPECS
-from releng_tool.defs import RPK_GIT_SUBMODULES
-from releng_tool.defs import RPK_GIT_VERIFY_REVISION
-from releng_tool.defs import RPK_INSTALL_DEFS
-from releng_tool.defs import RPK_INSTALL_ENV
-from releng_tool.defs import RPK_INSTALL_OPTS
-from releng_tool.defs import RPK_INSTALL_TYPE
-from releng_tool.defs import RPK_INTERNAL
-from releng_tool.defs import RPK_LICENSE
-from releng_tool.defs import RPK_LICENSE_FILES
-from releng_tool.defs import RPK_NO_EXTRACTION
-from releng_tool.defs import RPK_PREFIX
-from releng_tool.defs import RPK_PYTHON_INTERPRETER
-from releng_tool.defs import RPK_REVISION
-from releng_tool.defs import RPK_SITE
-from releng_tool.defs import RPK_SKIP_REMOTE_CONFIG
-from releng_tool.defs import RPK_SKIP_REMOTE_SCRIPTS
-from releng_tool.defs import RPK_STRIP_COUNT
-from releng_tool.defs import RPK_TYPE
-from releng_tool.defs import RPK_VCS_TYPE
-from releng_tool.defs import RPK_VERSION
+from releng_tool.defs import Rpk
 from releng_tool.defs import VcsType
 from releng_tool.packages import PkgKeyType
 from releng_tool.packages import pkg_cache_key
@@ -51,9 +13,9 @@ from releng_tool.packages.exceptions import RelengToolConflictingLocalSrcsPath
 from releng_tool.packages.exceptions import RelengToolCyclicPackageDependency
 from releng_tool.packages.exceptions import RelengToolInvalidPackageKeyValue
 from releng_tool.packages.exceptions import RelengToolInvalidPackageScript
+from releng_tool.packages.exceptions import RelengToolMissingPackageRevision
 from releng_tool.packages.exceptions import RelengToolMissingPackageScript
 from releng_tool.packages.exceptions import RelengToolMissingPackageSite
-from releng_tool.packages.exceptions import RelengToolMissingPackageVersion
 from releng_tool.packages.exceptions import RelengToolUnknownExtractType
 from releng_tool.packages.exceptions import RelengToolUnknownInstallType
 from releng_tool.packages.exceptions import RelengToolUnknownPackageType
@@ -75,6 +37,7 @@ from releng_tool.util.string import interpret_string
 from releng_tool.util.string import interpret_strings
 from releng_tool.util.string import interpret_zero_to_one_strings
 import pickle
+import pprint
 import os
 import traceback
 
@@ -123,44 +86,51 @@ class RelengPackageManager:
         self._load_dvcs_cache()
 
         # register expected types for each configuration
-        self._register_conf(RPK_AUTOTOOLS_AUTORECONF, PkgKeyType.BOOL)
-        self._register_conf(RPK_BUILD_DEFS, PkgKeyType.DICT_STR_STR)
-        self._register_conf(RPK_BUILD_ENV, PkgKeyType.DICT_STR_STR)
-        self._register_conf(RPK_BUILD_OPTS, PkgKeyType.DICT_STR_STR_OR_STRS)
-        self._register_conf(RPK_BUILD_SUBDIR, PkgKeyType.STR)
-        self._register_conf(RPK_CONF_DEFS, PkgKeyType.DICT_STR_STR)
-        self._register_conf(RPK_CONF_ENV, PkgKeyType.DICT_STR_STR)
-        self._register_conf(RPK_CONF_OPTS, PkgKeyType.DICT_STR_STR_OR_STRS)
-        self._register_conf(RPK_DEPS, PkgKeyType.STRS)
-        self._register_conf(RPK_DEVMODE_IGNORE_CACHE, PkgKeyType.BOOL)
-        self._register_conf(RPK_DEVMODE_REVISION, PkgKeyType.STR)
-        self._register_conf(RPK_EXTENSION, PkgKeyType.STR)
-        self._register_conf(RPK_EXTERNAL, PkgKeyType.BOOL)
-        self._register_conf(RPK_EXTOPT, PkgKeyType.DICT)
-        self._register_conf(RPK_EXTRACT_TYPE, PkgKeyType.STR)
-        self._register_conf(RPK_FIXED_JOBS, PkgKeyType.INT_POSITIVE)
-        self._register_conf(RPK_GIT_CONFIG, PkgKeyType.DICT_STR_STR)
-        self._register_conf(RPK_GIT_DEPTH, PkgKeyType.INT_NONNEGATIVE)
-        self._register_conf(RPK_GIT_REFSPECS, PkgKeyType.STRS)
-        self._register_conf(RPK_GIT_SUBMODULES, PkgKeyType.BOOL)
-        self._register_conf(RPK_GIT_VERIFY_REVISION, PkgKeyType.BOOL)
-        self._register_conf(RPK_INSTALL_DEFS, PkgKeyType.DICT_STR_STR)
-        self._register_conf(RPK_INSTALL_ENV, PkgKeyType.DICT_STR_STR)
-        self._register_conf(RPK_INSTALL_OPTS, PkgKeyType.DICT_STR_STR_OR_STRS)
-        self._register_conf(RPK_INSTALL_TYPE, PkgKeyType.STR)
-        self._register_conf(RPK_INTERNAL, PkgKeyType.BOOL)
-        self._register_conf(RPK_LICENSE, PkgKeyType.STRS)
-        self._register_conf(RPK_LICENSE_FILES, PkgKeyType.STRS)
-        self._register_conf(RPK_NO_EXTRACTION, PkgKeyType.BOOL)
-        self._register_conf(RPK_PREFIX, PkgKeyType.STR)
-        self._register_conf(RPK_PYTHON_INTERPRETER, PkgKeyType.STR)
-        self._register_conf(RPK_REVISION, PkgKeyType.STR)
-        self._register_conf(RPK_SITE, PkgKeyType.STR)
-        self._register_conf(RPK_SKIP_REMOTE_CONFIG, PkgKeyType.BOOL)
-        self._register_conf(RPK_SKIP_REMOTE_SCRIPTS, PkgKeyType.BOOL)
-        self._register_conf(RPK_STRIP_COUNT, PkgKeyType.INT_NONNEGATIVE)
-        self._register_conf(RPK_TYPE, PkgKeyType.STR)
-        self._register_conf(RPK_VCS_TYPE, PkgKeyType.STR)
+        self._register_conf(Rpk.AUTOTOOLS_AUTORECONF, PkgKeyType.BOOL)
+        self._register_conf(Rpk.BUILD_DEFS, PkgKeyType.DICT_STR_STR)
+        self._register_conf(Rpk.BUILD_ENV, PkgKeyType.DICT_STR_STR)
+        self._register_conf(Rpk.BUILD_OPTS, PkgKeyType.DICT_STR_STR_OR_STRS)
+        self._register_conf(Rpk.BUILD_SUBDIR, PkgKeyType.STR)
+        self._register_conf(Rpk.CMAKE_NOINSTALL, PkgKeyType.BOOL)
+        self._register_conf(Rpk.CONF_DEFS, PkgKeyType.DICT_STR_STR)
+        self._register_conf(Rpk.CONF_ENV, PkgKeyType.DICT_STR_STR)
+        self._register_conf(Rpk.CONF_OPTS, PkgKeyType.DICT_STR_STR_OR_STRS)
+        self._register_conf(Rpk.DEPS, PkgKeyType.STRS)
+        self._register_conf(Rpk.DEVMODE_IGNORE_CACHE, PkgKeyType.BOOL)
+        self._register_conf(Rpk.DEVMODE_REVISION, PkgKeyType.STR)
+        self._register_conf(Rpk.EXTENSION, PkgKeyType.STR)
+        self._register_conf(Rpk.EXTERNAL, PkgKeyType.BOOL)
+        self._register_conf(Rpk.EXTOPT, PkgKeyType.DICT)
+        self._register_conf(Rpk.EXTRACT_TYPE, PkgKeyType.STR)
+        self._register_conf(Rpk.FETCH_OPTS, PkgKeyType.DICT_STR_STR_OR_STRS)
+        self._register_conf(Rpk.FIXED_JOBS, PkgKeyType.INT_POSITIVE)
+        self._register_conf(Rpk.GIT_CONFIG, PkgKeyType.DICT_STR_STR)
+        self._register_conf(Rpk.GIT_DEPTH, PkgKeyType.INT_NONNEGATIVE)
+        self._register_conf(Rpk.GIT_REFSPECS, PkgKeyType.STRS)
+        self._register_conf(Rpk.GIT_SUBMODULES, PkgKeyType.BOOL)
+        self._register_conf(Rpk.GIT_VERIFY_REVISION, PkgKeyType.BOOL)
+        self._register_conf(Rpk.INSTALL_DEFS, PkgKeyType.DICT_STR_STR)
+        self._register_conf(Rpk.INSTALL_ENV, PkgKeyType.DICT_STR_STR)
+        self._register_conf(Rpk.INSTALL_OPTS, PkgKeyType.DICT_STR_STR_OR_STRS)
+        self._register_conf(Rpk.INSTALL_TYPE, PkgKeyType.STR)
+        self._register_conf(Rpk.INTERNAL, PkgKeyType.BOOL)
+        self._register_conf(Rpk.LICENSE, PkgKeyType.STRS)
+        self._register_conf(Rpk.LICENSE_FILES, PkgKeyType.STRS)
+        self._register_conf(Rpk.NO_EXTRACTION, PkgKeyType.BOOL)
+        self._register_conf(Rpk.PREFIX, PkgKeyType.STR)
+        self._register_conf(Rpk.PYTHON_INTERPRETER, PkgKeyType.STR)
+        self._register_conf(Rpk.REVISION, PkgKeyType.STR)
+        self._register_conf(Rpk.SITE, PkgKeyType.STR)
+        self._register_conf(Rpk.SKIP_REMOTE_CONFIG, PkgKeyType.BOOL)
+        self._register_conf(Rpk.SKIP_REMOTE_SCRIPTS, PkgKeyType.BOOL)
+        self._register_conf(Rpk.STRIP_COUNT, PkgKeyType.INT_NONNEGATIVE)
+        self._register_conf(Rpk.TYPE, PkgKeyType.STR)
+        self._register_conf(Rpk.VCS_TYPE, PkgKeyType.STR)
+        self._register_conf(Rpk.VERSION, PkgKeyType.STR)
+
+        # sanity check that check option is properly registered
+        for key in Rpk:
+            assert key in self._key_types, 'key {} is missing'.format(key)
 
     def load(self, names):
         """
@@ -221,8 +191,7 @@ class RelengPackageManager:
                     })
                 elif dep not in pkgs:
                     if dep not in names_left:
-                        verbose(
-                            'adding implicitly defined package: {}'.format(dep))
+                        verbose('adding implicitly defined package: {}', dep)
                         names_left.append(dep)
 
                     if pkg not in final_deps:
@@ -232,7 +201,7 @@ class RelengPackageManager:
                     pkg.deps.append(pkgs[dep])
             extend_script_env(self.script_env, env)
 
-        # for packages which have a dependency but have not been binded yet,
+        # for packages which have a dependency but have not been bound yet,
         # bind the dependencies now
         for pkg, deps in final_deps.items():
             for dep in deps:
@@ -252,7 +221,7 @@ class RelengPackageManager:
                 })
         debug('sorted packages)')
         for pkg in sorted_pkgs:
-            debug(' {}'.format(pkg.name))
+            debug(' {}', pkg.name)
 
         return sorted_pkgs
 
@@ -279,8 +248,8 @@ class RelengPackageManager:
             RelengToolInvalidPackageConfiguration: when an error has been
                                                     detected loading the package
         """
-        verbose('loading package: {}'.format(name))
-        debug('script {}'.format(script))
+        verbose('loading package: {}', name)
+        debug('script {}', script)
         opts = self.opts
 
         if not os.path.isfile(script):
@@ -301,33 +270,37 @@ class RelengPackageManager:
         self._active_package = name
         self._active_env = env
 
-        # version
+        # prepare helper expand values
+        expand_extra = {
+        }
+
+        # version/revision extraction first
         #
-        # Always check version first since it will be the most commonly used
-        # package field -- rather initially fail on a simple field first (for
-        # new packages and/or developers) than breaking on a possibly more
-        # complex field below.
-        key = pkg_key(name, RPK_VERSION)
-        if key not in env or not env[key]:
-            raise RelengToolMissingPackageVersion({
-                'pkg_name': name,
-                'pkg_key': key,
-            })
-        pkg_version = interpret_string(env[key])
-        if pkg_version is None:
-            raise RelengToolInvalidPackageKeyValue({
-                'pkg_name': name,
-                'pkg_key': key,
-                'expected_type': 'string',
-            })
+        # Attempt to check the version first since it will be the most commonly
+        # used package field -- rather initially fail on a simple field first
+        # (for new packages and/or developers) than breaking on a possibly more
+        # complex field below. Note that the version field is optional, in cases
+        # where a package type does not need a version entry (e.g. sites which
+        # do not require a version value for fetching or there is not revision
+        # value to use instead).
+        #
+        # Note that when in development mode, the development-mode revision
+        # (if any is set) needs to be checked as well. This value may override
+        # the package's version value.
+
+        # version
+        pkg_version = self._fetch(Rpk.VERSION)
+
+        if not pkg_version:
+            pkg_version = ''
+
+        pkg_version_key = pkg_key(name, Rpk.VERSION)
+        expand_extra[pkg_version_key] = pkg_version
 
         # development mode revision
-        #
-        # Always check for a development-mode revision after the version, as
-        # this value may override the package's version value if development
-        # mode is enabled.
         pkg_has_devmode_option = False
-        pkg_devmode_revision = self._fetch(RPK_DEVMODE_REVISION)
+        pkg_devmode_revision = self._fetch(Rpk.DEVMODE_REVISION,
+            allow_expand=True, expand_extra=expand_extra)
 
         if pkg_devmode_revision:
             pkg_has_devmode_option = True
@@ -337,54 +310,25 @@ class RelengPackageManager:
 
             if opts.devmode:
                 pkg_version = pkg_devmode_revision
-
-        # prepare helper expand values
-        expand_extra = {
-            key: pkg_version,
-        }
-
-        # archive extraction strip count
-        pkg_strip_count = self._fetch(RPK_STRIP_COUNT,
-            default=DEFAULT_STRIP_COUNT)
-
-        # build subdirectory
-        pkg_build_subdir = self._fetch(RPK_BUILD_SUBDIR)
-
-        # dependencies
-        deps = self._fetch(RPK_DEPS, default=[])
-
-        # ignore cache
-        pkg_devmode_ignore_cache = self._fetch(RPK_DEVMODE_IGNORE_CACHE)
-
-        # extension (override)
-        pkg_filename_ext = self._fetch(RPK_EXTENSION)
-
-        # extract type
-        pkg_extract_type = self._fetch(RPK_EXTRACT_TYPE)
-        if pkg_extract_type:
-            pkg_extract_type = pkg_extract_type.lower()
-
-            if pkg_extract_type not in self.registry.extract_types:
-                raise RelengToolUnknownExtractType({
-                    'pkg_name': name,
-                    'pkg_key': key,
-                })
-
-        # is-external
-        pkg_is_external = self._fetch(RPK_EXTERNAL)
-
-        # is-internal
-        pkg_is_internal = self._fetch(RPK_INTERNAL)
-
-        # no extraction
-        pkg_no_extraction = self._fetch(RPK_NO_EXTRACTION)
+                expand_extra[pkg_version_key] = pkg_version
 
         # revision
         if opts.revision_override and name in opts.revision_override:
             pkg_revision = opts.revision_override[name]
         else:
-            pkg_revision = self._fetch(RPK_REVISION,
+            pkg_revision = self._fetch(Rpk.REVISION,
                 allow_expand=True, expand_extra=expand_extra)
+        if opts.devmode and pkg_devmode_revision:
+            pkg_revision = pkg_devmode_revision
+        elif not pkg_revision:
+            pkg_revision = pkg_version
+
+        # site / vcs-site detection
+        #
+        # After extracted required version information, the site / VCS type
+        # needs to be checked next. This will allow the manage to early detect
+        # if a version/revision field is required, and fail early if we have
+        # not detected one from above.
 
         # site
         if opts.sites_override and name in opts.sites_override:
@@ -394,36 +338,12 @@ class RelengPackageManager:
             # source when experiencing network connectivity issues.
             pkg_site = opts.sites_override[name]
         else:
-            pkg_site = self._fetch(RPK_SITE,
+            pkg_site = self._fetch(Rpk.SITE,
                 allow_expand=True, expand_extra=expand_extra)
 
-        # skip any remote configuration
-        pkg_skip_remote_config = self._fetch(RPK_SKIP_REMOTE_CONFIG)
-
-        # skip any remote scripts
-        pkg_skip_remote_scripts = self._fetch(RPK_SKIP_REMOTE_SCRIPTS)
-
-        # type
-        pkg_type = None
-        pkg_type_raw = self._fetch(RPK_TYPE)
-        if pkg_type_raw:
-            pkg_type_raw = pkg_type_raw.lower()
-            if pkg_type_raw in PackageType:
-                pkg_type = pkg_type_raw
-            elif pkg_type_raw in self.registry.package_types:
-                pkg_type = pkg_type_raw
-            else:
-                raise RelengToolUnknownPackageType({
-                    'pkg_name': name,
-                    'pkg_key': key,
-                })
-
-        if not pkg_type:
-            pkg_type = PackageType.SCRIPT
-
         # vcs-type
         pkg_vcs_type = None
-        pkg_vcs_type_raw = self._fetch(RPK_VCS_TYPE)
+        pkg_vcs_type_raw = self._fetch(Rpk.VCS_TYPE)
         if pkg_vcs_type_raw:
             pkg_vcs_type_raw = pkg_vcs_type_raw.lower()
             if pkg_vcs_type_raw in VcsType:
@@ -433,7 +353,7 @@ class RelengPackageManager:
             else:
                 raise RelengToolUnknownVcsType({
                     'pkg_name': name,
-                    'pkg_key': key,
+                    'pkg_key': pkg_key(name, Rpk.VCS_TYPE),
                 })
 
         if not pkg_vcs_type:
@@ -461,6 +381,9 @@ class RelengPackageManager:
                 elif site_lc.startswith('hg+'):
                     pkg_site = pkg_site[3:]
                     pkg_vcs_type = VcsType.HG
+                elif site_lc.startswith('rsync+'):
+                    pkg_site = pkg_site[6:]
+                    pkg_vcs_type = VcsType.RSYNC
                 elif site_lc.startswith('scp+'):
                     pkg_site = pkg_site[4:]
                     pkg_vcs_type = VcsType.SCP
@@ -475,24 +398,100 @@ class RelengPackageManager:
                 pkg_vcs_type = VcsType.NONE
 
         if pkg_vcs_type == VcsType.LOCAL:
-            warn('package using local content: {}'.format(name))
+            warn('package using local content: {}', name)
+
+        # check if the detected vcs type needs a revision, and fail if we do
+        # not have one
+        if not pkg_revision and pkg_vcs_type in (
+                VcsType.BZR,
+                VcsType.CVS,
+                VcsType.GIT,
+                VcsType.HG,
+                VcsType.SVN,
+                ):
+            raise RelengToolMissingPackageRevision({
+                'pkg_name': name,
+                'pkg_key1': pkg_key(name, Rpk.VERSION),
+                'pkg_key2': pkg_key(name, Rpk.REVISION),
+                'vcs_type': pkg_vcs_type,
+            })
+
+        # archive extraction strip count
+        pkg_strip_count = self._fetch(Rpk.STRIP_COUNT,
+            default=DEFAULT_STRIP_COUNT)
+
+        # build subdirectory
+        pkg_build_subdir = self._fetch(Rpk.BUILD_SUBDIR)
+
+        # dependencies
+        deps = self._fetch(Rpk.DEPS, default=[])
+
+        # ignore cache
+        pkg_devmode_ignore_cache = self._fetch(Rpk.DEVMODE_IGNORE_CACHE)
+
+        # extension (override)
+        pkg_filename_ext = self._fetch(Rpk.EXTENSION)
+
+        # extract type
+        pkg_extract_type = self._fetch(Rpk.EXTRACT_TYPE)
+        if pkg_extract_type:
+            pkg_extract_type = pkg_extract_type.lower()
+
+            if pkg_extract_type not in self.registry.extract_types:
+                raise RelengToolUnknownExtractType({
+                    'pkg_name': name,
+                    'pkg_key': pkg_key(name, Rpk.EXTRACT_TYPE),
+                })
+
+        # is-external
+        pkg_is_external = self._fetch(Rpk.EXTERNAL)
+
+        # is-internal
+        pkg_is_internal = self._fetch(Rpk.INTERNAL)
+
+        # no extraction
+        pkg_no_extraction = self._fetch(Rpk.NO_EXTRACTION)
+
+        # skip any remote configuration
+        pkg_skip_remote_config = self._fetch(Rpk.SKIP_REMOTE_CONFIG)
+
+        # skip any remote scripts
+        pkg_skip_remote_scripts = self._fetch(Rpk.SKIP_REMOTE_SCRIPTS)
+
+        # type
+        pkg_type = None
+        pkg_type_raw = self._fetch(Rpk.TYPE)
+        if pkg_type_raw:
+            pkg_type_raw = pkg_type_raw.lower()
+            if pkg_type_raw in PackageType:
+                pkg_type = pkg_type_raw
+            elif pkg_type_raw in self.registry.package_types:
+                pkg_type = pkg_type_raw
+            else:
+                raise RelengToolUnknownPackageType({
+                    'pkg_name': name,
+                    'pkg_key': pkg_key(name, Rpk.TYPE),
+                })
+
+        if not pkg_type:
+            pkg_type = PackageType.SCRIPT
 
         # ######################################################################
 
         # git configuration options for a repository
-        pkg_git_config = self._fetch(RPK_GIT_CONFIG)
+        pkg_git_config = self._fetch(Rpk.GIT_CONFIG)
 
         # git-depth
-        pkg_git_depth = self._fetch(RPK_GIT_DEPTH)
+        pkg_git_depth = self._fetch(Rpk.GIT_DEPTH)
 
         # git-refspecs
-        pkg_git_refspecs = self._fetch(RPK_GIT_REFSPECS)
+        pkg_git_refspecs = self._fetch(Rpk.GIT_REFSPECS)
 
         # git-submodules
-        pkg_git_submodules = self._fetch(RPK_GIT_SUBMODULES)
+        pkg_git_submodules = self._fetch(Rpk.GIT_SUBMODULES)
 
         # git-verify
-        pkg_git_verify_revision = self._fetch(RPK_GIT_VERIFY_REVISION)
+        pkg_git_verify_revision = self._fetch(Rpk.GIT_VERIFY_REVISION)
 
         # ######################################################################
 
@@ -501,8 +500,8 @@ class RelengPackageManager:
             if pkg_is_external == pkg_is_internal:
                 raise RelengToolConflictingConfiguration({
                     'pkg_name': name,
-                    'pkg_key1': pkg_key(name, RPK_EXTERNAL),
-                    'pkg_key2': pkg_key(name, RPK_INTERNAL),
+                    'pkg_key1': pkg_key(name, Rpk.EXTERNAL),
+                    'pkg_key2': pkg_key(name, Rpk.INTERNAL),
                     'desc': 'package flagged as external and internal',
                 })
         elif pkg_is_external is not None:
@@ -520,13 +519,14 @@ class RelengPackageManager:
                 VcsType.CVS,
                 VcsType.GIT,
                 VcsType.HG,
+                VcsType.RSYNC,
                 VcsType.SCP,
                 VcsType.SVN,
                 VcsType.URL,
                 ):
             raise RelengToolMissingPackageSite({
                 'pkg_name': name,
-                'pkg_key': key,
+                'pkg_key': pkg_key(name, Rpk.SITE),
                 'vcs_type': pkg_vcs_type,
             })
 
@@ -540,7 +540,12 @@ class RelengPackageManager:
         # find possible extension for a cache file
         #
         # non-dvcs's will be always gzip-tar'ed.
-        if pkg_vcs_type in (VcsType.BZR, VcsType.CVS, VcsType.SVN):
+        if pkg_vcs_type in (
+                VcsType.BZR,
+                VcsType.CVS,
+                VcsType.RSYNC,
+                VcsType.SVN,
+                ):
             cache_ext = 'tgz'
         # dvcs's will not have an extension type
         elif is_pkg_dvcs:
@@ -566,7 +571,24 @@ class RelengPackageManager:
                     __, cache_ext = interpret_stem_extension(basename)
 
         # prepare package container and directory locations
-        pkg_nv = '{}-{}'.format(name, pkg_version)
+        #
+        # The container folder for a package will typically be a combination of
+        # a package's name plus version. If no version is set, the container
+        # will be only use the package's name. We try to use the version entry
+        # when possible to help manage multiple versions of output (e.g. to
+        # avoid conflicts when bumping versions).
+        #
+        # When the version value is used, we will attempt to cleanup/minimize
+        # the version to help provide the container a more "sane" path. For
+        # instance, if a version references a path-styled branch names (e.g.
+        # `bugfix/my-bug`, we want to avoid promoting a container name which
+        # can result in a sub-directory being made (e.g. `pkg-bugfix/my-bug/`).
+        if pkg_version:
+            pkg_nv = '{}-{}'.format(name, ''.join(
+                x if (x.isalnum() or x in '-._') else '_' for x in pkg_version))
+        else:
+            pkg_nv = name
+
         pkg_build_output_dir = os.path.join(opts.build_dir, pkg_nv)
         pkg_def_dir = os.path.abspath(os.path.join(script, os.pardir))
         if pkg_vcs_type == VcsType.LOCAL:
@@ -589,10 +611,6 @@ class RelengPackageManager:
             pkg_cache_file = os.path.join(opts.dl_dir, pkg_nv + '.' + cache_ext)
         else:
             pkg_cache_file = os.path.join(opts.dl_dir, pkg_nv)
-        if opts.devmode and pkg_devmode_revision:
-            pkg_revision = pkg_devmode_revision
-        elif not pkg_revision:
-            pkg_revision = pkg_version
 
         # Select sources (like CMake-based projects) may wish to be using
         # out-of-source tree builds. For supported project types, adjust the
@@ -602,6 +620,24 @@ class RelengPackageManager:
             pkg_build_output_dir = os.path.join(
                 pkg_build_output_dir, 'releng-output')
 
+        # determine the build tree for a package
+        #
+        # A build tree (introduced for the libfoo-exec action), tracks the
+        # directory where build commands would typically be executed for a
+        # package on a host system. In most cases, this will be set to the
+        # same path as `pkg_build_dir` (or the sub-directory, if provided);
+        # however, some package types may have a better working directory
+        # for build commands. For example, CMake projects will generate a
+        # build package in an out-of-source directory (e.g.
+        # `pkg_build_output_dir`), which is a better make to issue commands
+        # such as "cmake --build .".
+        if pkg_type == PackageType.CMAKE:
+            pkg_build_tree = pkg_build_output_dir
+        elif pkg_build_subdir:
+            pkg_build_tree = pkg_build_subdir
+        else:
+            pkg_build_tree = pkg_build_dir
+
         # determine the package directory for this package
         #
         # Typically, a package's "cache directory" will be stored in the output
@@ -624,8 +660,8 @@ class RelengPackageManager:
             # if the cache content is stored in another container, use it
             elif ckey in self._dvcs_cache:
                 pkg_cache_dirname = self._dvcs_cache[ckey]
-                verbose('alternative cache path for package: {} -> {}'.format(
-                    name, pkg_cache_dirname))
+                verbose('alternative cache path for package: {} -> {}',
+                    name, pkg_cache_dirname)
 
             # track ckey entry to point to our cache container
             #
@@ -648,6 +684,7 @@ class RelengPackageManager:
         pkg.build_dir = pkg_build_dir
         pkg.build_output_dir = pkg_build_output_dir
         pkg.build_subdir = pkg_build_subdir
+        pkg.build_tree = pkg_build_tree
         pkg.cache_dir = pkg_cache_dir
         pkg.cache_file = pkg_cache_file
         pkg.def_dir = pkg_def_dir
@@ -676,12 +713,13 @@ class RelengPackageManager:
         for env in (os.environ, env):
             env[pkg_key(name, 'BUILD_DIR')] = pkg_build_dir
             env[pkg_key(name, 'BUILD_OUTPUT_DIR')] = pkg_build_output_dir
+            env[pkg_key(name, 'DEFDIR')] = pkg_def_dir
             env[pkg_key(name, 'NAME')] = name
             env[pkg_key(name, 'REVISION')] = pkg_revision
-        os.environ[pkg_key(name, RPK_VERSION)] = pkg_version
+        os.environ[pkg_key(name, Rpk.VERSION)] = pkg_version
 
         # (internals)
-        prefix = '.stage_'
+        prefix = '.releng_tool-stage-'
         outdir = pkg.build_output_dir
         pkg._ff_bootstrap = os.path.join(outdir, prefix + 'bootstrap')
         pkg._ff_build = os.path.join(outdir, prefix + 'build')
@@ -692,6 +730,18 @@ class RelengPackageManager:
         pkg._ff_patch = os.path.join(outdir, prefix + 'patch')
         pkg._ff_post = os.path.join(outdir, prefix + 'post')
 
+        # dump package attributes if running in a debug mode
+        if opts.debug:
+            info = {}
+            for key, value in pkg.__dict__.items():
+                if not key.startswith('_'):
+                    info[key] = value
+
+            debug('''package-data: {}
+==============================
+{}
+==============================''', name, pprint.pformat(info))
+
         return pkg, env, deps
 
     def _apply_postinit_options(self, pkg):
@@ -721,15 +771,19 @@ class RelengPackageManager:
 
         # extension modifiers
         if pkg.ext_modifiers is None:
-            pkg.ext_modifiers = self._fetch(RPK_EXTOPT)
+            pkg.ext_modifiers = self._fetch(Rpk.EXTOPT)
+
+        # fetch options
+        if pkg.fetch_opts is None:
+            pkg.fetch_opts = self._fetch(Rpk.FETCH_OPTS)
 
         # fixed jobs
         if pkg.fixed_jobs is None:
-            pkg.fixed_jobs = self._fetch(RPK_FIXED_JOBS)
+            pkg.fixed_jobs = self._fetch(Rpk.FIXED_JOBS)
 
         # install type
         if pkg.install_type is None:
-            pkg_install_type_raw = self._fetch(RPK_INSTALL_TYPE)
+            pkg_install_type_raw = self._fetch(Rpk.INSTALL_TYPE)
             if pkg_install_type_raw:
                 pkg_install_type_raw = pkg_install_type_raw.lower()
                 if pkg_install_type_raw in PackageInstallType:
@@ -737,20 +791,20 @@ class RelengPackageManager:
                 else:
                     raise RelengToolUnknownInstallType({
                         'pkg_name': pkg.name,
-                        'pkg_key': pkg_key(pkg.name, RPK_INSTALL_TYPE),
+                        'pkg_key': pkg_key(pkg.name, Rpk.INSTALL_TYPE),
                     })
 
         # license
         if pkg.license is None:
-            pkg.license = self._fetch(RPK_LICENSE)
+            pkg.license = self._fetch(Rpk.LICENSE)
 
         # license files
         if pkg.license_files is None:
-            pkg.license_files = self._fetch(RPK_LICENSE_FILES)
+            pkg.license_files = self._fetch(Rpk.LICENSE_FILES)
 
         # prefix
         if pkg.prefix is None:
-            pkg.prefix = self._fetch(RPK_PREFIX)
+            pkg.prefix = self._fetch(Rpk.PREFIX)
 
         # ######################################################################
         # (package type - shared)
@@ -758,39 +812,39 @@ class RelengPackageManager:
 
         # package-type build definitions
         if pkg.build_defs is None:
-            pkg.build_defs = self._fetch(RPK_BUILD_DEFS)
+            pkg.build_defs = self._fetch(Rpk.BUILD_DEFS)
 
         # package-type build environment options
         if pkg.build_env is None:
-            pkg.build_env = self._fetch(RPK_BUILD_ENV)
+            pkg.build_env = self._fetch(Rpk.BUILD_ENV)
 
         # package-type build options
         if pkg.build_opts is None:
-            pkg.build_opts = self._fetch(RPK_BUILD_OPTS)
+            pkg.build_opts = self._fetch(Rpk.BUILD_OPTS)
 
         # package-type configuration definitions
         if pkg.conf_defs is None:
-            pkg.conf_defs = self._fetch(RPK_CONF_DEFS)
+            pkg.conf_defs = self._fetch(Rpk.CONF_DEFS)
 
         # package-type configuration environment options
         if pkg.conf_env is None:
-            pkg.conf_env = self._fetch(RPK_CONF_ENV)
+            pkg.conf_env = self._fetch(Rpk.CONF_ENV)
 
         # package-type configuration options
         if pkg.conf_opts is None:
-            pkg.conf_opts = self._fetch(RPK_CONF_OPTS)
+            pkg.conf_opts = self._fetch(Rpk.CONF_OPTS)
 
         # package-type installation definitions
         if pkg.install_defs is None:
-            pkg.install_defs = self._fetch(RPK_INSTALL_DEFS)
+            pkg.install_defs = self._fetch(Rpk.INSTALL_DEFS)
 
         # package-type installation environment options
         if pkg.install_env is None:
-            pkg.install_env = self._fetch(RPK_INSTALL_ENV)
+            pkg.install_env = self._fetch(Rpk.INSTALL_ENV)
 
         # package-type installation options
         if pkg.install_opts is None:
-            pkg.install_opts = self._fetch(RPK_INSTALL_OPTS)
+            pkg.install_opts = self._fetch(Rpk.INSTALL_OPTS)
 
         # ######################################################################
         # (package type - autotools)
@@ -798,7 +852,15 @@ class RelengPackageManager:
 
         # autotools autoreconf flag
         if pkg.autotools_autoreconf is None:
-            pkg.autotools_autoreconf = self._fetch(RPK_AUTOTOOLS_AUTORECONF)
+            pkg.autotools_autoreconf = self._fetch(Rpk.AUTOTOOLS_AUTORECONF)
+
+        # ######################################################################
+        # (package type - cmake)
+        # ######################################################################
+
+        # cmake noinstall flag
+        if pkg.cmake_noinstall is None:
+            pkg.cmake_noinstall = self._fetch(Rpk.CMAKE_NOINSTALL)
 
         # ######################################################################
         # (package type - python)
@@ -806,7 +868,7 @@ class RelengPackageManager:
 
         # python interpreter
         if pkg.python_interpreter is None:
-            pkg_python_interpreter = self._fetch(RPK_PYTHON_INTERPRETER)
+            pkg_python_interpreter = self._fetch(Rpk.PYTHON_INTERPRETER)
             pkg.python_interpreter = pkg_python_interpreter
 
     def load_remote_configuration(self, pkg):
@@ -866,8 +928,8 @@ class RelengPackageManager:
                                                     detected loading any of the
                                                     package's extended options
         """
-        verbose('finalize package configuration: {}'.format(pkg.name))
-        debug('script {}'.format(script))
+        verbose('finalize package configuration: {}', pkg.name)
+        debug('script {}', script)
 
         if not os.path.isfile(script):
             raise RelengToolMissingPackageScript({
@@ -916,7 +978,6 @@ class RelengPackageManager:
         Raises:
             RelengToolInvalidPackageKeyValue: value type is invalid for the key
         """
-        assert key in self._key_types
 
         self._active_key = key
         type_ = self._key_types[key]
@@ -932,48 +993,56 @@ class RelengPackageManager:
                 'expected_type': type_,
             })
 
-        if pkg_key_ in self._active_env:
+        # check if this package key has been explicitly overridden; if so,
+        # use its contents for the raw value to process
+        raw_value = self.opts.injected_kv.get(pkg_key_, None)
+
+        # if no raw value was injected, pull the key's value (if any) from the
+        # active environment
+        if raw_value is None:
+            raw_value = self._active_env.get(pkg_key_, None)
+
+        if raw_value is not None:
             if type_ == PkgKeyType.BOOL:
-                value = self._active_env[pkg_key_]
+                value = raw_value
                 if not isinstance(value, bool):
                     raise_kv_exception('bool')
             elif type_ == PkgKeyType.DICT:
-                value = self._active_env[pkg_key_]
+                value = raw_value
                 if allow_expand:
                     value = expand(value, expand_extra)
                 if not isinstance(value, dict):
                     raise_kv_exception('dictionary')
             elif type_ == PkgKeyType.DICT_STR_STR:
-                value = interpret_dictionary_strings(self._active_env[pkg_key_])
+                value = interpret_dictionary_strings(raw_value)
                 if allow_expand:
                     value = expand(value, expand_extra)
                 if value is None:
                     raise_kv_exception('dict(str,str)')
             elif type_ == PkgKeyType.DICT_STR_STR_OR_STRS:
-                value = interpret_zero_to_one_strings(
-                    self._active_env[pkg_key_])
+                value = interpret_zero_to_one_strings(raw_value)
                 if allow_expand:
                     value = expand(value, expand_extra)
                 if value is None:
                     raise_kv_exception('dict(str,str) or string(s)')
             elif type_ == PkgKeyType.STR:
-                value = interpret_string(self._active_env[pkg_key_])
+                value = interpret_string(raw_value)
                 if allow_expand:
                     value = expand(value, expand_extra)
                 if value is None:
                     raise_kv_exception('string')
             elif type_ == PkgKeyType.STRS:
-                value = interpret_strings(self._active_env[pkg_key_])
+                value = interpret_strings(raw_value)
                 if allow_expand:
                     value = expand(value, expand_extra)
                 if value is None:
                     raise_kv_exception('string(s)')
             elif type_ == PkgKeyType.INT_NONNEGATIVE:
-                value = self._active_env[pkg_key_]
+                value = raw_value
                 if not isinstance(value, int) or value < 0:
                     raise_kv_exception('non-negative int')
             elif type_ == PkgKeyType.INT_POSITIVE:
-                value = self._active_env[pkg_key_]
+                value = raw_value
                 if not isinstance(value, int) or value <= 0:
                     raise_kv_exception('positive int')
             else:
@@ -1039,5 +1108,6 @@ class RelengPackageManager:
             key: the key
             type_: the expected configuration type
         """
-        assert key not in self._key_types
+
+        assert key not in self._key_types, 'key {} is registered'.format(key)
         self._key_types[key] = type_
