@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2022 releng-tool
 
 class RelengPackage:
     """
@@ -9,12 +9,14 @@ class RelengPackage:
 
     Args:
         name: the name of the package
+        version: the version of the package
 
     Attributes:
         asc_file: file containing ascii-armored data to validate this package
         build_dir: directory for a package's buildable content
         build_output_dir: build output directory for the package process
         build_subdir: override for a package's buildable content (if applicable)
+        build_tree: the build tree directory for a package
         cache_dir: cache directory for the package (if applicable)
         cache_file: cache file for the package (if applicable)
         def_dir: directory for the package definition
@@ -22,6 +24,7 @@ class RelengPackage:
         devmode_ignore_cache: whether or not cache files should be ignored
         ext_modifiers: extension-defined modifiers (dict)
         extract_type: extraction type override (for extensions, if applicable)
+        fetch_opts: fetch options (if applicable)
         fixed_jobs: fixed job count for this specific package
         git_config: git config options to apply (if applicable)
         git_depth: git fetch depth (if applicable)
@@ -58,23 +61,30 @@ class RelengPackage:
         install_opts: package-type installation option overrides
         (package type - autotools)
         autotools_autoreconf: flag to invoke autoreconf
+        (package type - cmake)
+        cmake_noinstall: flag to disable the install stage for a cmake project
         (other - python)
         python_interpreter: python interpreter to invoke stages with
     """
     def __init__(self, name, version):
         self.name = name
-        self.nv = '{}-{}'.format(name, version)
         self.version = version
+        if version:
+            self.nv = '{}-{}'.format(name, version)
+        else:
+            self.nv = self.name
         # (commons)
         self.asc_file = None
         self.build_dir = None
         self.build_subdir = None
         self.build_output_dir = None
+        self.build_tree = None
         self.cache_dir = None
         self.cache_file = None
         self.def_dir = None
         self.deps = []
         self.devmode_ignore_cache = None
+        self.fetch_opts = None
         self.fixed_jobs = None
         self.has_devmode_option = None
         self.hash_file = None
@@ -105,6 +115,8 @@ class RelengPackage:
         self.install_opts = None
         # (package type - autotools)
         self.autotools_autoreconf = None
+        # (package type - cmake)
+        self.cmake_noinstall = None
         # (other - git)
         self.git_config = None
         self.git_depth = None
