@@ -30,7 +30,8 @@ except ImportError:
 # filename for the statistics database
 STATISTICS_NAME = 'statistics.dat'
 
-class RelengStats():
+
+class RelengStats:
     """
     statistics tracking
 
@@ -302,7 +303,6 @@ class RelengStats():
             fig_total.savefig(dur_pdf_total)
 
             # close/cleanup figures
-            for fig in figs:
-                plt.close()
+            plt.close()
         else:
             debug('duration statistics plot not supported')
