@@ -1,17 +1,19 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2022 releng-tool
 
 from __future__ import unicode_literals
 from contextlib import contextmanager
-from distutils.dir_util import DistutilsFileError
-from distutils.dir_util import copy_tree
+from releng_tool.util.compat import CopyTreeError
+from releng_tool.util.compat import copy_tree
 from releng_tool.util.log import debug
 from releng_tool.util.log import err
 from releng_tool.util.log import is_verbose
 from releng_tool.util.log import verbose
 from releng_tool.util.log import warn
 from runpy import run_path
+from shutil import Error as ShutilError
 from shutil import copy2
+from shutil import copyfileobj
 from shutil import move
 import errno
 import os
@@ -41,13 +43,51 @@ class FailedToPrepareBaseDirectoryError(Exception):
     """
     raised when a base directory could not be prepared
     """
-    pass
 
 class FailedToPrepareWorkingDirectoryError(Exception):
     """
     raised when a working directory could not be prepared
     """
-    pass
+
+def cat(file, *args):
+    """
+    concatenate files and print on the standard output
+
+    Attempts to read one or more files provided to this call. For each file, it
+    will be read and printed out to the standard output.
+
+    An example when using in the context of script helpers is as follows:
+
+    .. code-block:: python
+
+        releng_cat('my-file')
+
+    Args:
+        file: the file
+        *args (optional): additional files to include
+
+    Returns:
+        ``True`` if all the files exists and were printed to the standard
+        output; ``False`` if one or more files could not be read
+    """
+
+    files = []
+    files.append(file)
+    for arg in args:
+        files.append(arg)
+
+    for f in files:
+        if not os.path.isfile(f):
+            return False
+
+    try:
+        for filename in files:
+            with open(filename, 'r') as f:
+                copyfileobj(f, sys.stdout)
+
+        return True
+    except OSError:
+        return False
 
 def ensure_dir_exists(dir_, quiet=False):
     """
@@ -80,10 +120,10 @@ def ensure_dir_exists(dir_, quiet=False):
     try:
         os.makedirs(dir_)
     except OSError as e:
-        if e.errno != errno.EEXIST:
+        if e.errno != errno.EEXIST or not os.path.isdir(dir_):
             if not quiet:
-                err('unable to create directory: ' + dir_)
-                err('    {}'.format(e))
+                err('unable to create directory: {}\n'
+                    '    {}', dir_, e)
             return False
     return True
 
@@ -155,9 +195,17 @@ def execute(args, cwd=None, env=None, env_update=None, quiet=None,
         SystemExit: if the execution operation fails with ``critical=True``
     """
 
-    rv = _execute(args, cwd=cwd, env=env, env_update=env_update,
-        quiet=quiet, critical=critical, poll=poll, capture=capture)
-    return (rv == 0)
+    rv = _execute(
+        args,
+        capture=capture,
+        critical=critical,
+        cwd=cwd,
+        env=env,
+        env_update=env_update,
+        poll=poll,
+        quiet=quiet,
+    )
+    return rv == 0
 
 def execute_rv(command, *args, **kwargs):
     """
@@ -200,11 +248,15 @@ def execute_rv(command, *args, **kwargs):
     """
 
     out = []
-    rv = _execute([command] + list(args),
+    rv = _execute(
+        [command] + list(args),
+        capture=out,
+        critical=False,
         cwd=kwargs.get('cwd'),
         env=kwargs.get('env'),
         env_update=kwargs.get('env_update'),
-        capture=out, quiet=True, critical=False)
+        quiet=True,
+    )
     return rv, '\n'.join(out)
 
 def _execute(args, cwd=None, env=None, env_update=None, quiet=None,
@@ -298,11 +350,11 @@ def _execute(args, cwd=None, env=None, env_update=None, quiet=None,
         if final_env and sys.version_info[0] < 3:
             debug('detected python 2.7; sanity checking environment variables')
             for k, v in final_env.items():
-                if isinstance(v, unicode): # noqa: F821
+                if isinstance(v, unicode): # pylint: disable=E0602 # noqa: F821
                     final_env[k] = v.encode('ascii', 'replace')
 
         if is_verbose():
-            debug('(wd) {}'.format(cwd if cwd else os.getcwd()))
+            debug('(wd) {}', cwd if cwd else os.getcwd())
             cmd_str = _cmd_args_to_str(args)
             verbose('invoking: ' + cmd_str)
             sys.stdout.flush()
@@ -319,9 +371,15 @@ def _execute(args, cwd=None, env=None, env_update=None, quiet=None,
                 bufsize = 1
                 universal_newlines = True
 
-            proc = subprocess.Popen(args, bufsize=bufsize,
-                stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
-                cwd=cwd, env=final_env, universal_newlines=universal_newlines)
+            proc = subprocess.Popen(
+                args,
+                bufsize=bufsize,
+                cwd=cwd,
+                env=final_env,
+                stderr=subprocess.STDOUT,
+                stdout=subprocess.PIPE,
+                universal_newlines=universal_newlines,
+            )
 
             if bufsize == 0:
                 line = bytearray()
@@ -355,16 +413,24 @@ def _execute(args, cwd=None, env=None, env_update=None, quiet=None,
                 if not cmd_str:
                     cmd_str = _cmd_args_to_str(args)
 
-                err('unable to execute command: ' + cmd_str)
-                err('    {}'.format(e))
+                err('unable to execute command: {}\n'
+                    '    {}', cmd_str, e)
 
     if rv != 0:
-        if args:
+        if critical:
+            if args:
+                cmd_str = _cmd_args_to_str(args)
+            else:
+                cmd_str = '<empty>'
+            err('failed to issue command: ' + cmd_str)
+
+            # trigger a hard stop
+            sys.exit(-1)
+        elif args:
             debug('failed to issue last command')
         else:
             debug('failed to issue an empty command')
-        if critical:
-            sys.exit(-1)
+
     return rv
 
 def _cmd_args_to_str(args):
@@ -430,8 +496,8 @@ def generate_temp_dir(dir_=None):
             path_remove(dir_)
         except OSError as e:
             if e.errno != errno.ENOENT:
-                warn('unable to cleanup temporary directory: ' + dir_)
-                warn('    {}'.format(e))
+                warn('unable to cleanup temporary directory: {}\n'
+                     '    {}', dir_, e)
 
 @contextmanager
 def interim_working_dir(dir_):
@@ -499,10 +565,10 @@ def interpret_stem_extension(basename):
         a 2-tuple (stem, extension)
     """
     if not basename:
-        return (None, None)
+        return None, None
 
     if '.' not in basename:
-        return (basename, None)
+        return basename, None
 
     stem, ext = basename.split('.', 1)
     while '.' in ext:
@@ -512,7 +578,43 @@ def interpret_stem_extension(basename):
         part, ext = ext.split('.', 1)
         stem = '{}.{}'.format(stem, part)
 
-    return (stem, ext)
+    return stem, ext
+
+def ls(dir_):
+    """
+    list a directory's contents
+
+    Attempts to read a directory for its contents and prints this information
+    to the configured standard output stream.
+
+    An example when using in the context of script helpers is as follows:
+
+    .. code-block:: python
+
+        releng_ls('my-dir/')
+
+    Args:
+        dir_: the directory
+
+    Returns:
+        ``True`` if the directory could be read and its contents have been
+        printed to the standard output; ``False`` if the directory could not
+        be read
+    """
+
+    if not os.path.isdir(dir_):
+        return False
+
+    try:
+        for entry in os.listdir(dir_):
+            if os.path.isdir(os.path.join(dir_, entry)):
+                print(entry + '/')
+            else:
+                print(entry)
+
+        return True
+    except OSError:
+        return False
 
 def opt_file(file):
     """
@@ -544,7 +646,7 @@ def opt_file(file):
 
     return file, exists
 
-def path_copy(src, dst, quiet=False, critical=True):
+def path_copy(src, dst, quiet=False, critical=True, dst_dir=False):
     """
     copy a file or directory into a target file or directory
 
@@ -586,6 +688,7 @@ def path_copy(src, dst, quiet=False, critical=True):
         dst: the destination directory or file\\* (\\*if ``src`` is a file)
         quiet (optional): whether or not to suppress output
         critical (optional): whether or not to stop execution on failure
+        dst_dir (optional): force hint that the destination is a directory
 
     Returns:
         ``True`` if the copy has completed with no error; ``False`` if the copy
@@ -599,22 +702,30 @@ def path_copy(src, dst, quiet=False, critical=True):
 
     try:
         if os.path.isfile(src):
-            parent_dir = os.path.dirname(dst)
-            if parent_dir and not os.path.isdir(parent_dir):
-                ensure_dir_exists(parent_dir, quiet=quiet)
-            copy2(src, dst)
-            success = True
+            attempt_copy = True
+
+            if dst_dir:
+                base_dir = dst
+            else:
+                base_dir = os.path.dirname(dst)
+
+            if base_dir and not os.path.isdir(base_dir):
+                attempt_copy = ensure_dir_exists(base_dir, quiet=quiet)
+
+            if attempt_copy:
+                copy2(src, dst)
+                success = True
         elif os.path.exists(src):
             copy_tree(src, dst)
             success = True
         else:
             errmsg = 'source does not exist: {}'.format(src)
-    except (DistutilsFileError, IOError) as e:
+    except (CopyTreeError, IOError, ShutilError) as e:
         errmsg = str(e)
 
     if not quiet and errmsg:
         err('unable to copy source contents to target location\n'
-            '    {}'.format(errmsg))
+            '    {}', errmsg)
 
     if not success and critical:
         sys.exit(-1)
@@ -710,8 +821,8 @@ def path_move(src, dst, quiet=False, critical=True):
         dst_dir = os.path.realpath(dst)
         if dst_dir.startswith(src_dir):
             if not quiet:
-                err('unable to move source contents to target location')
-                err('    attempt to move directory into a child subdirectory')
+                err('unable to move source contents to target location\n'
+                    '    attempt to move directory into a child subdirectory')
             if critical:
                 sys.exit(-1)
             return False
@@ -728,8 +839,8 @@ def path_move(src, dst, quiet=False, critical=True):
         except Exception as e:
             success = False
             if not quiet:
-                err('unable to move source contents to target location')
-                err('    {}'.format(e))
+                err('unable to move source contents to target location\n'
+                    '    {}', e)
 
     if not success and critical:
         sys.exit(-1)
@@ -816,6 +927,10 @@ def path_remove(path, quiet=False):
         ``True`` if the path was removed or does not exist; ``False`` if the
         path could not be removed from the system
     """
+
+    if not os.path.exists(path):
+        return True
+
     try:
         if os.path.isdir(path) and not os.path.islink(path):
             _path_remove_dir(path)
@@ -824,8 +939,8 @@ def path_remove(path, quiet=False):
     except OSError as e:
         if e.errno != errno.ENOENT:
             if not quiet:
-                err('unable to remove path: ' + path)
-                err('    {}'.format(e))
+                err('unable to remove path: {}\n'
+                    '    {}', path, e)
             return False
 
     return True
@@ -966,8 +1081,6 @@ def prepare_definitions(defs, prefix=None):
 
             if prefix:
                 key = prefix + key
-            else:
-                key = key
 
             if val:
                 final.append('{}={}'.format(key, val))
@@ -1036,10 +1149,12 @@ def run_script(script, globals_, subject=None, catch=True):
         try:
             result = run_path(script, init_globals=globals_)
         except Exception as e:
-            err(traceback.format_exc())
-            err('error running {}{}script: {}'.format(
-                subject, subject and ' ', script))
-            err('    {}'.format(e))
+            err('{}\n'
+                'error running {}{}script: {}\n'
+                '    {}',
+                traceback.format_exc(),
+                subject, subject and ' ', script,
+                e)
             return None
 
     return result
@@ -1073,7 +1188,7 @@ def touch(file):
         if parent_dir and not os.path.isdir(parent_dir):
             ensure_dir_exists(parent_dir)
 
-        with open(file, 'a'):
+        with open(file, 'ab'):
             os.utime(file, None)
         return True
     except OSError:
