@@ -1,17 +1,17 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2022 releng-tool
 
 import os
 
 try:
-    basestring
+    basestring  # pylint: disable=E0601
 except NameError:
     basestring = str
 
 try:
     from collections.abc import Sequence
 except ImportError:
-    from collections import Sequence
+    from collections import Sequence  # pylint: disable=W1512
 
 def expand(obj, kv=None):
     """
