@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2022 releng-tool
 
 from contextlib import contextmanager
 from difflib import unified_diff
@@ -128,6 +128,29 @@ def prepare_workdir():
     with generate_temp_dir() as work_dir:
         yield work_dir
 
+@contextmanager
+def redirect_stderr(new_target=None):
+    """
+    temporarily redirect stderr to another instance
+
+    This call will temporarily redirect stderr to the provided instance
+    until the end of the context, where the previous `stderr` target will be
+    restored.
+
+    Args:
+        new_target (optional): the instance to map stderr to
+    """
+
+    if not new_target:
+        new_target = StringIO()
+
+    _ = sys.stderr
+    try:
+        sys.stderr = new_target
+        yield new_target
+    finally:
+        sys.stderr = _
+
 @contextmanager
 def redirect_stdout(new_target=None):
     """
@@ -186,13 +209,26 @@ def find_test_base():
 
     test_base = os.path.dirname(os.path.realpath(__file__))
 
-    if not os.path.exists(os.path.join(test_base, 'templates')):
+    def templates_exist(test_base):
+        return os.path.exists(os.path.join(test_base, 'templates'))
+
+    if not templates_exist(test_base):
         test_base = os.path.dirname(os.path.abspath(sys.argv[0]))
+        if not templates_exist(test_base):
+            # python 2.7 may not always be able to find the test directory at
+            # all stages of test; try to rely on a provided tox ini directory if
+            # running in a tox environment
+            if sys.version_info < (3, 0) and 'TOX_INI_DIR' in os.environ:
+                root_dir = os.environ['TOX_INI_DIR']
+                test_base = os.path.join(root_dir, 'tests')
+
+            if not templates_exist(test_base):
+                raise RuntimeError('unable to find test base directory')
 
     return test_base
 
 class RelengToolTestSuite(unittest.TestSuite):
-    def run(self, result):
+    def run(self, result, debug=False):
         """
         a releng-tool helper test suite
 
@@ -205,10 +241,11 @@ class RelengToolTestSuite(unittest.TestSuite):
 
         Args:
             result: the test result object to populate
+            debug (optional): debug flag to ignore error collection
 
         Returns:
             the test result object
         """
-        rv = unittest.TestSuite.run(self, result)
+        rv = unittest.TestSuite.run(self, result, debug)
         sys.stdout.flush()
         return rv
