@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2021 releng-tool
+# Copyright 2021-2022 releng-tool
 
 from releng_tool.defs import GlobalAction
 from releng_tool.util.io import execute
@@ -27,8 +27,8 @@ class TestToolGit(TestSiteToolBase):
         self.assertTrue(rv)
 
     def test_tool_git_basic_commit(self):
-        hash = self._create_commit()
-        self.defconfig_add('VERSION', hash)
+        hash_ = self._create_commit()
+        self.defconfig_add('VERSION', hash_)
         rv = self.engine.run()
         self.assertTrue(rv)
 
@@ -38,6 +38,17 @@ class TestToolGit(TestSiteToolBase):
         rv = self.engine.run()
         self.assertTrue(rv)
 
+    def test_tool_git_branch_forward_slash(self):
+        # create a commit on a new branch with a forward slash
+        TMP_BRANCH = 'container/test'
+        self._git_repo('checkout', '-b', TMP_BRANCH)
+        self._create_commit()
+        self._git_repo('checkout', DEFAULT_BRANCH)
+
+        self.defconfig_add('VERSION', TMP_BRANCH)
+        rv = self.engine.run()
+        self.assertTrue(rv)
+
     def test_tool_git_changing_revision(self):
         # verify pulling from an original tag
         first_hash = self._create_commit('first commit')
@@ -103,14 +114,14 @@ class TestToolGit(TestSiteToolBase):
         # create a commit on a new custom reference
         TMP_BRANCH = 'tmp'
         self._git_repo('checkout', '-b', TMP_BRANCH)
-        hash = self._create_commit()
-        self._git_repo('update-ref', 'refs/mycustomref/test', hash)
+        hash_ = self._create_commit()
+        self._git_repo('update-ref', 'refs/mycustomref/test', hash_)
         self._git_repo('checkout', DEFAULT_BRANCH)
         self._git_repo('branch', '-D', TMP_BRANCH)
 
         # first fetch attempt should fail since the custom reference will not be
         # fetched by default
-        self.defconfig_add('VERSION', hash)
+        self.defconfig_add('VERSION', hash_)
         rv = self.engine.run()
         self.assertFalse(rv)
 
@@ -121,8 +132,8 @@ class TestToolGit(TestSiteToolBase):
 
     def test_tool_git_depth_shallow_default(self):
         self._create_commit()
-        hash = self._create_commit()
-        self.defconfig_add('VERSION', hash)
+        hash_ = self._create_commit()
+        self.defconfig_add('VERSION', hash_)
 
         rv = self.engine.run()
         self.assertTrue(rv)
@@ -132,9 +143,9 @@ class TestToolGit(TestSiteToolBase):
 
     def test_tool_git_depth_shallow_disabled_config(self):
         self._create_commit()
-        hash = self._create_commit()
+        hash_ = self._create_commit()
         self.defconfig_add('GIT_DEPTH', 0)
-        self.defconfig_add('VERSION', hash)
+        self.defconfig_add('VERSION', hash_)
 
         rv = self.engine.run()
         self.assertTrue(rv)
@@ -146,8 +157,8 @@ class TestToolGit(TestSiteToolBase):
         self.engine.opts.quirks.append('releng.git.no_depth')
 
         self._create_commit()
-        hash = self._create_commit()
-        self.defconfig_add('VERSION', hash)
+        hash_ = self._create_commit()
+        self.defconfig_add('VERSION', hash_)
 
         rv = self.engine.run()
         self.assertTrue(rv)
@@ -156,9 +167,9 @@ class TestToolGit(TestSiteToolBase):
         self.assertNotEqual(int(rev_count), 1)
 
     def test_tool_git_depth_unshallow_initial(self):
-        hash = self._create_commit()
+        hash_ = self._create_commit()
         self._create_commit()
-        self.defconfig_add('VERSION', hash)
+        self.defconfig_add('VERSION', hash_)
 
         rv = self.engine.run()
         self.assertTrue(rv)
