@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
-# Copyright 2018-2021 releng-tool
+# Copyright 2018-2022 releng-tool
 
 from collections import OrderedDict
+from releng_tool.util.io import ensure_dir_exists
 from releng_tool.util.io import execute
 from releng_tool.util.io import interpret_stem_extension as ise
 from releng_tool.util.io import opt_file
@@ -12,6 +13,7 @@ from releng_tool.util.io import path_remove
 from releng_tool.util.io import prepare_arguments
 from releng_tool.util.io import prepare_definitions
 from releng_tool.util.io import prepend_shebang_interpreter as psi
+from releng_tool.util.io import run_script
 from releng_tool.util.io import touch
 from releng_tool.util.log import is_verbose
 from tests import compare_contents
@@ -83,6 +85,62 @@ class TestUtilIo(unittest.TestCase):
             cc4 = compare_contents(copied_file_b, target_ow)
             self.assertIsNone(cc4, cc4)
 
+            # attempt to copy a missing file
+            missing_file = os.path.join(work_dir, 'test-file-missing')
+            target = os.path.join(work_dir, 'container')
+            result = path_copy(missing_file, target, critical=False)
+            self.assertFalse(result)
+
+            # attempt to copy a missing file (critical)
+            with self.assertRaises(SystemExit):
+                path_copy(missing_file, target)
+
+            # attempt to copy a file to itself
+            src = copied_file_a
+            with self.assertRaises(SystemExit):
+                path_copy(src, src)
+
+            # attempt to copy a directory to itself in legacy python, to ensure
+            # `DistutilsFileError` is handled properly (windows only)
+            if sys.version_info < (3, 0) and sys.platform == 'win32':
+                with self.assertRaises(SystemExit):
+                    path_copy(work_dir, work_dir)
+
+            # force a directory target with a non-trailing path separator
+            force_src = os.path.join(work_dir, 'test-file-a')
+            self.assertExists(force_src)
+
+            force1 = os.path.join(work_dir, 'force1')
+            result = path_copy(force_src, force1, critical=False)
+            self.assertTrue(result)
+            self.assertTrue(os.path.isfile(force1))
+
+            force2 = os.path.join(work_dir, 'force2')
+            result = path_copy(force_src, force2, critical=False, dst_dir=True)
+            self.assertTrue(result)
+            self.assertTrue(os.path.isdir(force2))
+
+    def test_utilio_ensuredirexists(self):
+        with prepare_workdir() as work_dir:
+            result = ensure_dir_exists(work_dir)
+            self.assertTrue(result)
+
+            new_dir = os.path.join(work_dir, 'test1')
+            self.assertFalse(os.path.exists(new_dir))
+
+            result = ensure_dir_exists(new_dir)
+            self.assertTrue(result)
+            self.assertTrue(os.path.exists(new_dir))
+
+            new_file = os.path.join(work_dir, 'test2')
+            with open(new_file, 'ab'):
+                pass
+            self.assertTrue(os.path.isfile(new_file))
+
+            result = ensure_dir_exists(new_file)
+            self.assertFalse(result)
+            self.assertTrue(os.path.isfile(new_file))
+
     def test_utilio_execution(self):
         result = execute(None, quiet=True, critical=False)
         self.assertFalse(result)
@@ -362,8 +420,8 @@ class TestUtilIo(unittest.TestCase):
                 _('dir2', 'dir3'),
                 _('dir4', 'dir5', 'dir6'),
             ]
-            for dir in directories:
-                os.makedirs(dir)
+            for dir_ in directories:
+                os.makedirs(dir_)
 
             files = [
                 _('file1'),
@@ -416,6 +474,25 @@ class TestUtilIo(unittest.TestCase):
             removed = path_remove(path)
             self.assertTrue(removed)
 
+    def test_utilio_runscript(self):
+        with prepare_workdir() as work_dir:
+            valid_script = os.path.join(work_dir, 'valid')
+            with open(valid_script, 'a') as f:
+                f.write('test=1\n')
+
+            result = run_script(valid_script, {})
+            self.assertEqual(result['test'], 1)
+
+            invalid_script = os.path.join(work_dir, 'invalid')
+            with open(invalid_script, 'a') as f:
+                f.write('bad-line\n')
+
+            result = run_script(invalid_script, {})
+            self.assertIsNone(result)
+
+            with self.assertRaises(NameError):
+                run_script(invalid_script, {}, catch=False)
+
     def test_utilio_shebang_interpreter(self):
         si_dir = os.path.join(self.assets_dir, 'shebang-interpreter')
         si01 = [os.path.join(si_dir, 'interpreter')]
